@@ -30,7 +30,7 @@ Boston, MA 02111-1307, USA.  */
 #include <X11/ObjectP.h>
 #include <X11/CompositeP.h>
 #include <X11/Shell.h>
-#ifdef HAVE_X_WIDGETS
+#ifdef HAVE_WIDGETS
 #include "../src/EmacsManager.h"
 #endif
 #ifdef LWLIB_MENUBARS_LUCID
@@ -49,9 +49,15 @@ Boston, MA 02111-1307, USA.  */
 #include "../src/xmu.h"
 #include "xlwtabs.h"
 #endif
-
 
 
+#include <X11/Intrinsic.h>
+#include <X11/StringDefs.h>
+#include <X11/Shell.h>
+#include <X11/Xaw/Form.h>
+#include <X11/Xaw/Command.h>
+
+
 #ifdef LWLIB_MENUBARS_LUCID
 
 /* Menu callbacks */
@@ -142,9 +148,8 @@ xlw_create_popup_menu (widget_instance* instance)
   XtSetArg (al [0], XtNmenu, instance->info->val);
   XtSetArg (al [1], XtNhorizontal, False);
   widget = XtCreateManagedWidget ("popup", xlwMenuWidgetClass,
-				  popup_shell, al, 2);
-  XtAddCallback (widget, XtNselect, pick_hook, (XtPointer)instance);
-
+  			 popup_shell, al, 2);
+  XtAddCallback (widget, XtNselect, pick_hook, (XtPointer)instance);  
   return popup_shell;
 }
 #endif /* LWLIB_MENUBARS_LUCID */
@@ -158,13 +163,13 @@ xlw_scrollbar_callback (Widget widget, XtPointer closure, XtPointer call_data)
   XlwScrollBarCallbackStruct *data =
     (XlwScrollBarCallbackStruct *) call_data;
   scroll_event event_data;
-  scrollbar_values *val =
-    (scrollbar_values *) instance->info->val->scrollbar_data;
+  scrollbar_values *val;
   double percent;
 
   if (!instance || widget->core.being_destroyed)
     return;
 
+  val = (scrollbar_values *) instance->info->val->scrollbar_data;
   id = instance->info->id;
 
   percent = (double) (data->value - 1) / (double) (INT_MAX - 1);
@@ -319,13 +324,21 @@ xlw_update_scrollbar (widget_instance *instance, Widget widget,
 #ifdef LWLIB_TABS_LUCID
 /* tab control
    
-   lwlib is such an incredible hairy crock. I just cannot believe
+   [[ lwlib is such an incredible hairy crock. I just cannot believe
    it! There are random dependencies between functions, there is a
    total lack of genericity, even though it initially appears to be
    generic. It should all be junked and begun again. Building tabs are
    an example - in theory we should be able to reuse a lot of the
    general stuff because we want to put labels of whatever toolkit we
-   are using in the tab. Instead we have to hack it by hand. */
+   are using in the tab. Instead we have to hack it by hand. ]]
+   While lwlib is a hairy crock, whoever wrote that seems to misunderstand
+   Falk's tab control widget.  The tab control widget has *two* kinds of
+   children: *widgets*, which all occupy a *single* pane below the row of
+   tabs---this is where the labels created in build_tabs_in_widget go, and
+   *gadgets*, the tabs themselves, which do *not* draw themselves, but
+   rather are drawn by the control.  In fact, in XEmacs the true widget
+   children are *never* visible!  So this case is not a problem in the
+   design of lwlib, but rather of Falk's widget. -- sjt */
 static void
 xlw_tab_control_callback (Widget w, XtPointer client_data, XtPointer call_data)
 {
@@ -375,9 +388,8 @@ xlw_create_tab_control (widget_instance *instance)
   widget_value* val = instance->info->val;
 
   XtSetArg (al [ac], XtNsensitive, val->enabled);		ac++;
-  XtSetArg (al [ac], XtNmappedWhenManaged, FALSE);	ac++;
+  XtSetArg (al [ac], XtNmappedWhenManaged, False);		ac++;
   XtSetArg (al [ac], XtNorientation, XtorientHorizontal);	ac++;
-  XtSetArg (al [ac], XtNresizable, False);			ac++;
 
   /* add any args the user supplied for creation time */
   lw_add_value_args_to_args (val, al, &ac);
@@ -396,15 +408,22 @@ static void build_tabs_in_widget (widget_instance* instance, Widget widget,
 				  widget_value* val)
 {
   widget_value* cur = val;
+  Arg al[1];
+
+  /* Children are always invisible, don't permit resizing. */
+  XtSetArg (al[0], XtNresizable, False);
+
   for (cur = val; cur; cur = cur->next)
     {
       if (cur->value)
 	{
+	  Widget w;
 #ifdef LWLIB_WIDGETS_MOTIF
-	  xm_create_label (widget, cur);
+	  w = xm_create_label (widget, cur);
 #else
-	  xaw_create_label (widget, cur);
+	  w = xaw_create_label (widget, cur);
 #endif
+	  XtSetValues (w, al, 1);
 	}
       cur->change = NO_CHANGE;
     }
@@ -415,7 +434,7 @@ xlw_update_tab_control (widget_instance* instance, Widget widget, widget_value*
 {
   Widget* children;
   unsigned int num_children;
-  Dimension i;
+  int i;
   widget_value *cur = 0;
 
   XtRemoveAllCallbacks (widget, XtNcallback);
@@ -432,7 +451,7 @@ xlw_update_tab_control (widget_instance* instance, Widget widget, widget_value*
   children = XtCompositeChildren (widget, &num_children);
   if (children)
     {
-      for (i = 0, cur = val->contents; i < (int) num_children; i++)
+      for (i = 0, cur = val->contents; i < num_children; i++)
 	{
 	  if (!cur)
 	    abort ();
@@ -456,7 +475,7 @@ xlw_update_tab_control (widget_instance* instance, Widget widget, widget_value*
 }
 #endif /* LWLIB_TABS_LUCID */
 
-#ifdef HAVE_X_WIDGETS
+#ifdef HAVE_WIDGETS
 static Widget
 xlw_create_clip_window (widget_instance *instance)
 {
@@ -496,7 +515,7 @@ xlw_creation_table [] =
 #ifdef LWLIB_TABS_LUCID
   {"tab-control",	xlw_create_tab_control},
 #endif
-#ifdef HAVE_X_WIDGETS
+#ifdef HAVE_WIDGETS
   {"clip-window", xlw_create_clip_window},
 #endif
   {NULL, NULL}
@@ -524,7 +543,7 @@ lw_lucid_widget_p (Widget widget)
       XtClass (((CompositeWidget)widget)->composite.children [0])
 	== xlwMenuWidgetClass;
 #endif
-#ifdef HAVE_X_WIDGETS
+#ifdef HAVE_WIDGETS
   if (the_class == emacsManagerWidgetClass)
     return True;
 #endif
@@ -535,12 +554,12 @@ void
 xlw_update_one_widget (widget_instance* instance, Widget widget,
 		       widget_value* val, Boolean deep_p)
 {
-  WidgetClass class_ = XtClass (widget);
+  WidgetClass class = XtClass (widget);
 
   if (0)
     ;
 #ifdef LWLIB_MENUBARS_LUCID
-  else if (class_ == xlwMenuWidgetClass)
+  else if (class == xlwMenuWidgetClass)
     {
       XlwMenuWidget mw;
       Arg al [1];
@@ -553,13 +572,13 @@ xlw_update_one_widget (widget_instance* instance, Widget widget,
     }
 #endif
 #ifdef LWLIB_SCROLLBARS_LUCID
-  else if (class_ == xlwScrollBarWidgetClass)
+  else if (class == xlwScrollBarWidgetClass)
     {
       xlw_update_scrollbar (instance, widget, val);
     }
 #endif
 #ifdef LWLIB_TABS_LUCID
-  else if (class_ == tabsWidgetClass)
+  else if (class == tabsWidgetClass)
     {
       xlw_update_tab_control (instance, widget, val);
     }
