@@ -1,6 +1,5 @@
 /* Database access routines
    Copyright (C) 1996, William M. Perry
-   Copyright (C) 2001, 2002 Ben Wing.
 
 This file is part of XEmacs.
 
@@ -24,13 +23,12 @@ Boston, MA 02111-1307, USA.  */
 /* Written by Bill Perry */
 /* Substantially rewritten by Martin Buchholz */
 /* db 2.x support added by Andreas Jaeger */
-/* Mule-ized 6-22-00 Ben Wing */
 
 #include <config.h>
 #include "lisp.h"
-
 #include "sysfile.h"
 #include "buffer.h"
+#include <errno.h>
 
 #ifndef HAVE_DATABASE
 #error HAVE_DATABASE not defined!!
@@ -45,16 +43,24 @@ Boston, MA 02111-1307, USA.  */
 /* glibc 2.1 doesn't have this problem with DB 2.x */
 #if !(defined __GLIBC__ && __GLIBC_MINOR__ >= 1)
 #ifdef HAVE_INTTYPES_H
+#ifndef __BIT_TYPES_DEFINED__
 #define __BIT_TYPES_DEFINED__
+#endif
 #include <inttypes.h>
-#ifndef __FreeBSD__
+#if !HAVE_U_INT8_T
 typedef uint8_t  u_int8_t;
+#endif
+#if !HAVE_U_INT16_T
 typedef uint16_t u_int16_t;
+#endif
+#if !HAVE_U_INT32_T
 typedef uint32_t u_int32_t;
+#endif
 #ifdef WE_DONT_NEED_QUADS
+#if !HAVE_U_INT64_T
 typedef uint64_t u_int64_t;
+#endif
 #endif /* WE_DONT_NEED_QUADS */
-#endif /* __FreeBSD__ */
 #endif /* HAVE_INTTYPES_H */
 #endif /* !(defined __GLIBC__ && __GLIBC_MINOR__ >= 1) */
 /* Berkeley DB wants __STDC__ to be defined; else if does `#define const' */
@@ -80,7 +86,12 @@ Lisp_Object Qqueue;
 Lisp_Object Qdbm;
 #endif /* HAVE_DBM */
 
+#ifdef MULE
+/* #### The following should be settable on a per-database level.
+   But the whole coding-system infrastructure should be rewritten someday.
+   We really need coding-system aliases. -- martin */
 Lisp_Object Vdatabase_coding_system;
+#endif
 
 Lisp_Object Qdatabasep;
 
@@ -111,11 +122,13 @@ struct Lisp_Database
   DB *db_handle;
 #endif
   DB_FUNCS *funcs;
+#ifdef MULE
   Lisp_Object coding_system;
+#endif
 };
 
 #define XDATABASE(x) XRECORD (x, database, Lisp_Database)
-#define wrap_database(p) wrap_record (p, database)
+#define XSETDATABASE(x, p) XSETRECORD (x, p, database)
 #define DATABASEP(x) RECORDP (x, database)
 #define CHECK_DATABASE(x) CHECK_RECORD (x, database)
 #define CONCHECK_DATABASE(x) CONCHECK_RECORD (x, database)
@@ -124,7 +137,7 @@ struct Lisp_Database
 #define CHECK_LIVE_DATABASE(db) do {					\
   CHECK_DATABASE (db);							\
   if (!DATABASE_LIVE_P (XDATABASE(db)))					\
-    invalid_operation ("Attempting to access closed database", db);	\
+    signal_simple_error ("Attempting to access closed database", db);	\
 } while (0)
 
 
@@ -144,15 +157,12 @@ allocate_database (void)
   db->access_ = 0;
   db->mode = 0;
   db->dberrno = 0;
-  db->coding_system = Qnil;
+#ifdef MULE
+  db->coding_system = Fget_coding_system (Qbinary);
+#endif
   return db;
 }
 
-static const struct memory_description database_description[] = {
-  { XD_LISP_OBJECT, offsetof (struct Lisp_Database, fname) },
-  { XD_END}
-};
-
 static Lisp_Object
 mark_database (Lisp_Object object)
 {
@@ -163,20 +173,22 @@ mark_database (Lisp_Object object)
 static void
 print_database (Lisp_Object obj, Lisp_Object printcharfun, int escapeflag)
 {
+  char buf[64];
   Lisp_Database *db = XDATABASE (obj);
 
   if (print_readably)
-    printing_unreadable_object ("#<database 0x%x>", db->header.uid);
+    error ("printing unreadable object #<database 0x%x>", db->header.uid);
 
-  write_fmt_string_lisp (printcharfun, "#<database \"%s\" (%s/%s/",
-			 3, db->fname, db->funcs->get_type (db),
-			 db->funcs->get_subtype (db));
-
-  write_fmt_string (printcharfun, "%s) 0x%x>",
-		    (!DATABASE_LIVE_P (db)    ? "closed"    :
-		     (db->access_ & O_WRONLY) ? "writeonly" :
-		     (db->access_ & O_RDWR)   ? "readwrite" : "readonly"),
-		    db->header.uid);
+  write_c_string ("#<database \"", printcharfun);
+  print_internal (db->fname, printcharfun, 0);
+  sprintf (buf, "\" (%s/%s/%s) 0x%x>",
+	   (char *) string_data (XSYMBOL (db->funcs->get_type (db))->name),
+	   (char *) string_data (XSYMBOL (db->funcs->get_subtype (db))->name),
+	   (!DATABASE_LIVE_P (db)    ? "closed"    :
+	    (db->access_ & O_WRONLY) ? "writeonly" :
+	    (db->access_ & O_RDWR)   ? "readwrite" : "readonly"),
+	   db->header.uid);
+  write_c_string (buf, printcharfun);
 }
 
 static void
@@ -186,18 +198,18 @@ finalize_database (void *header, int for_disksave)
 
   if (for_disksave)
     {
-      invalid_operation
-	("Can't dump an emacs containing database objects",
-	 wrap_database (db));
+      Lisp_Object object;
+      XSETDATABASE (object, db);
+
+      signal_simple_error
+	("Can't dump an emacs containing database objects", object);
     }
   db->funcs->close (db);
 }
 
 DEFINE_LRECORD_IMPLEMENTATION ("database", database,
-			       0, /*dumpable-flag*/
                                mark_database, print_database,
-			       finalize_database, 0, 0, 
-			       database_description,
+			       finalize_database, 0, 0, 0,
 			       Lisp_Database);
 
 DEFUN ("close-database", Fclose_database, 1, 1, 0, /*
@@ -272,10 +284,8 @@ dbm_map (Lisp_Database *db, Lisp_Object func)
        keydatum = dbm_nextkey (db->dbm_handle))
     {
       valdatum = dbm_fetch (db->dbm_handle, keydatum);
-      key = make_ext_string (keydatum.dptr, keydatum.dsize,
-			     db->coding_system);
-      val = make_ext_string (valdatum.dptr, valdatum.dsize,
-			     db->coding_system);
+      key = make_string ((unsigned char *) keydatum.dptr, keydatum.dsize);
+      val = make_string ((unsigned char *) valdatum.dptr, valdatum.dsize);
       call2 (func, key, val);
     }
 }
@@ -285,14 +295,12 @@ dbm_get (Lisp_Database *db, Lisp_Object key)
 {
   datum keydatum, valdatum;
 
-  TO_EXTERNAL_FORMAT (LISP_STRING, key,
-		      ALLOCA, (keydatum.dptr, keydatum.dsize),
-		      db->coding_system);
+  keydatum.dptr = (char *) XSTRING_DATA (key);
+  keydatum.dsize = XSTRING_LENGTH (key);
   valdatum = dbm_fetch (db->dbm_handle, keydatum);
 
   return (valdatum.dptr
-	  ? make_ext_string (valdatum.dptr, valdatum.dsize,
-			     db->coding_system)
+	  ? make_string ((unsigned char *) valdatum.dptr, valdatum.dsize)
 	  : Qnil);
 }
 
@@ -302,12 +310,10 @@ dbm_put (Lisp_Database *db,
 {
   datum keydatum, valdatum;
 
-  TO_EXTERNAL_FORMAT (LISP_STRING, val,
-		      ALLOCA, (valdatum.dptr, valdatum.dsize),
-		      db->coding_system);
-  TO_EXTERNAL_FORMAT (LISP_STRING, key,
-		      ALLOCA, (keydatum.dptr, keydatum.dsize),
-		      db->coding_system);
+  valdatum.dptr = (char *) XSTRING_DATA (val);
+  valdatum.dsize = XSTRING_LENGTH (val);
+  keydatum.dptr = (char *) XSTRING_DATA (key);
+  keydatum.dsize = XSTRING_LENGTH (key);
 
   return !dbm_store (db->dbm_handle, keydatum, valdatum,
 		     NILP (replace) ? DBM_INSERT : DBM_REPLACE);
@@ -318,9 +324,8 @@ dbm_remove (Lisp_Database *db, Lisp_Object key)
 {
   datum keydatum;
 
-  TO_EXTERNAL_FORMAT (LISP_STRING, key,
-		      ALLOCA, (keydatum.dptr, keydatum.dsize),
-		      db->coding_system);
+  keydatum.dptr = (char *) XSTRING_DATA (key);
+  keydatum.dsize = XSTRING_LENGTH (key);
 
   return dbm_delete (db->dbm_handle, keydatum);
 }
@@ -407,9 +412,8 @@ berkdb_get (Lisp_Database *db, Lisp_Object key)
   xzero (keydatum);
   xzero (valdatum);
 
-  TO_EXTERNAL_FORMAT (LISP_STRING, key,
-		      ALLOCA, (keydatum.data, keydatum.size),
-		      db->coding_system);
+  keydatum.data = XSTRING_DATA (key);
+  keydatum.size = XSTRING_LENGTH (key);
 
 #if DB_VERSION_MAJOR == 1
   status = db->db_handle->get (db->db_handle, &keydatum, &valdatum, 0);
@@ -418,8 +422,8 @@ berkdb_get (Lisp_Database *db, Lisp_Object key)
 #endif /* DB_VERSION_MAJOR */
 
   if (!status)
-    return make_ext_string ((const Extbyte *) valdatum.data, valdatum.size,
-			    db->coding_system);
+    /* #### Not mule-ized! will crash! */
+    return make_string ((Bufbyte *) valdatum.data, valdatum.size);
 
 #if DB_VERSION_MAJOR == 1
   db->dberrno = (status == 1) ? -1 : errno;
@@ -443,12 +447,10 @@ berkdb_put (Lisp_Database *db,
   xzero (keydatum);
   xzero (valdatum);
 
-  TO_EXTERNAL_FORMAT (LISP_STRING, key,
-		      ALLOCA, (keydatum.data, keydatum.size),
-		      db->coding_system);
-  TO_EXTERNAL_FORMAT (LISP_STRING, val,
-		      ALLOCA, (valdatum.data, valdatum.size),
-		      db->coding_system);
+  keydatum.data = XSTRING_DATA   (key);
+  keydatum.size = XSTRING_LENGTH (key);
+  valdatum.data = XSTRING_DATA   (val);
+  valdatum.size = XSTRING_LENGTH (val);
 #if DB_VERSION_MAJOR == 1
   status = db->db_handle->put (db->db_handle, &keydatum, &valdatum,
  			       NILP (replace) ? R_NOOVERWRITE : 0);
@@ -471,9 +473,8 @@ berkdb_remove (Lisp_Database *db, Lisp_Object key)
   /* DB Version 2 requires DBT's to be zeroed before use. */
   xzero (keydatum);
 
-  TO_EXTERNAL_FORMAT (LISP_STRING, key,
-		      ALLOCA, (keydatum.data, keydatum.size),
-		      db->coding_system);
+  keydatum.data = XSTRING_DATA   (key);
+  keydatum.size = XSTRING_LENGTH (key);
 
 #if DB_VERSION_MAJOR == 1
   status = db->db_handle->del (db->db_handle, &keydatum, 0);
@@ -509,10 +510,9 @@ berkdb_map (Lisp_Database *db, Lisp_Object func)
        status == 0;
        status = dbp->seq (dbp, &keydatum, &valdatum, R_NEXT))
     {
-      key = make_ext_string (keydatum.data, keydatum.size,
-			     db->coding_system);
-      val = make_ext_string (valdatum.data, valdatum.size,
-			     db->coding_system);
+      /* #### Needs mule-izing */
+      key = make_string ((Bufbyte *) keydatum.data, keydatum.size);
+      val = make_string ((Bufbyte *) valdatum.data, valdatum.size);
       call2 (func, key, val);
     }
 #else
@@ -528,10 +528,9 @@ berkdb_map (Lisp_Database *db, Lisp_Object func)
 	 status == 0;
 	 status = dbcp->c_get (dbcp, &keydatum, &valdatum, DB_NEXT))
       {
-	key = make_ext_string ((const Extbyte *) keydatum.data, keydatum.size,
-			       db->coding_system);
-	val = make_ext_string ((const Extbyte *) valdatum.data, valdatum.size,
-			       db->coding_system);
+	/* #### Needs mule-izing */
+	key = make_string ((Bufbyte *) keydatum.data, keydatum.size);
+	val = make_string ((Bufbyte *) valdatum.data, valdatum.size);
 	call2 (func, key, val);
       }
     dbcp->c_close (dbcp);
@@ -581,18 +580,15 @@ Return the last error associated with DATABASE.
   return XDATABASE (database)->funcs->last_error (XDATABASE (database));
 }
 
-DEFUN ("open-database", Fopen_database, 1, 6, 0, /*
+DEFUN ("open-database", Fopen_database, 1, 5, 0, /*
 Return a new database object opened on FILE.
 Optional arguments TYPE and SUBTYPE specify the database type.
 Optional argument ACCESS specifies the access rights, which may be any
 combination of 'r' 'w' and '+', for read, write, and creation flags.
 Optional argument MODE gives the permissions to use when opening FILE,
 and defaults to 0755.
-Optional argument CODESYS specifies the coding system used to encode/decode
-data passed to/from the database, and defaults to the value of the
-variable `database-coding-system'.
 */
-       (file, type, subtype, access_, mode, codesys))
+       (file, type, subtype, access_, mode))
 {
   /* This function can GC */
   int modemask;
@@ -642,11 +638,6 @@ variable `database-coding-system'.
       modemask = XINT (mode);
     }
 
-  if (NILP (codesys))
-    codesys = Vdatabase_coding_system;
-
-  codesys = get_coding_system_for_text_file (Vdatabase_coding_system, 1);
-
 #ifdef HAVE_DBM
   if (NILP (type) || EQ (type, Qdbm))
     {
@@ -657,7 +648,6 @@ variable `database-coding-system'.
       db = allocate_database ();
       db->dbm_handle = dbase;
       db->funcs = &ndbm_func_block;
-      db->coding_system = codesys;
       goto db_done;
     }
 #endif /* HAVE_DBM */
@@ -682,7 +672,7 @@ variable `database-coding-system'.
 	real_subtype = DB_QUEUE;
 #endif
       else
-	invalid_constant ("Unsupported subtype", subtype);
+	signal_simple_error ("Unsupported subtype", subtype);
 
 #if DB_VERSION_MAJOR == 1
       dbase = dbopen (filename, accessmask, modemask, real_subtype, NULL);
@@ -719,7 +709,7 @@ variable `database-coding-system'.
       status = dbase->open (dbase, filename, NULL,
                             real_subtype, accessmask, modemask);
 #else /* DB_VERSION >= 4.1 */
-      /* You can't use DB_AUTO_COMMIT unless you have a txn environment. */
+      /* DB_AUTO_COMMIT requires transaction support, don't try it */
       status = dbase->open (dbase, NULL, filename, NULL, real_subtype,
 			    accessmask, modemask);
 #endif /* DB_VERSION < 4.1 */
@@ -736,12 +726,11 @@ variable `database-coding-system'.
       db = allocate_database ();
       db->db_handle = dbase;
       db->funcs = &berk_func_block;
-      db->coding_system = codesys;
       goto db_done;
     }
 #endif /* HAVE_BERKELEY_DB */
 
-  invalid_constant ("Unsupported database type", type);
+  signal_simple_error ("Unsupported database type", type);
   return Qnil;
 
  db_done:
@@ -750,7 +739,11 @@ variable `database-coding-system'.
   db->mode = modemask;
   db->access_ = accessmask;
 
-  return wrap_database (db);
+  {
+    Lisp_Object retval;
+    XSETDATABASE (retval, db);
+    return retval;
+  }
 }
 
 DEFUN ("put-database", Fput_database, 3, 4, 0, /*
@@ -817,19 +810,19 @@ syms_of_database (void)
 {
   INIT_LRECORD_IMPLEMENTATION (database);
 
-  DEFSYMBOL (Qdatabasep);
+  defsymbol (&Qdatabasep, "databasep");
 #ifdef HAVE_DBM
-  DEFSYMBOL (Qdbm);
+  defsymbol (&Qdbm, "dbm");
 #endif
 #ifdef HAVE_BERKELEY_DB
-  DEFSYMBOL (Qberkeley_db);
-  DEFSYMBOL (Qhash);
-  DEFSYMBOL (Qbtree);
-  DEFSYMBOL (Qrecno);
+  defsymbol (&Qberkeley_db, "berkeley-db");
+  defsymbol (&Qhash, "hash");
+  defsymbol (&Qbtree, "btree");
+  defsymbol (&Qrecno, "recno");
 #if DB_VERSION_MAJOR > 2
-  DEFSYMBOL (Qqueue);
+  defsymbol (&Qqueue, "queue");
 #endif
-  DEFSYMBOL (Qunknown);
+  defsymbol (&Qunknown, "unknown");
 #endif
 
   DEFSUBR (Fopen_database);
@@ -856,8 +849,12 @@ vars_of_database (void)
   Fprovide (Qberkeley_db);
 #endif
 
+#if 0 /* #### implement me! */
+#ifdef MULE
   DEFVAR_LISP ("database-coding-system", &Vdatabase_coding_system /*
-Default coding system used to convert data in database files.
+Coding system used to convert data in database files.
 */ );
-  Vdatabase_coding_system = Qnative;
+  Vdatabase_coding_system = Qnil;
+#endif
+#endif /* 0 */
 }
