@@ -1,6 +1,5 @@
 /* Hash tables.
    Copyright (C) 1992, 1993, 1994 Free Software Foundation, Inc.
-   Copyright (C) 2003 Ben Wing.
 
 This file is part of XEmacs.
 
@@ -21,33 +20,30 @@ Boston, MA 02111-1307, USA.  */
 
 /* Synched up with: Not in FSF. */
 
-/* Author: Lost in the mists of history.  At least back to Lucid 19.3,
-   circa Sep 1992. */
-
 #include <config.h>
 #include "lisp.h"
 #include "hash.h"
 
-#define NULL_ENTRY ((void *) 0xdeadbeef) /* -559038737 base 10 */
+#define NULL_ENTRY ((void *) 0xdeadbeef)
 
 #define COMFORTABLE_SIZE(size) (21 * (size) / 16)
 
 #define KEYS_DIFFER_P(old, new, testfun) \
   (((old) != (new)) && (!(testfun) || !(testfun) ((old),(new))))
 
-static void rehash (hentry *harray, struct hash_table *ht, Elemcount size);
+static void rehash (hentry *harray, struct hash_table *ht, hash_size_t size);
 
-Hashcode
-memory_hash (const void *xv, Bytecount size)
+unsigned long
+memory_hash (const void *xv, size_t size)
 {
-  Hashcode h = 0;
+  unsigned int h = 0;
   unsigned const char *x = (unsigned const char *) xv;
 
   if (!x) return 0;
 
   while (size--)
     {
-      Hashcode g;
+      unsigned int g;
       h = (h << 4) + *x++;
       if ((g = h & 0xf0000000) != 0)
 	h = (h ^ (g >> 24)) ^ g;
@@ -56,17 +52,17 @@ memory_hash (const void *xv, Bytecount size)
   return h;
 }
 
-Hashcode
+unsigned long
 string_hash (const char *xv)
 {
-  Hashcode h = 0;
+  unsigned int h = 0;
   unsigned const char *x = (unsigned const char *) xv;
 
   if (!x) return 0;
 
   while (*x)
     {
-      Hashcode g;
+      unsigned int g;
       h = (h << 4) + *x++;
       if ((g = h & 0xf0000000) != 0)
 	h = (h ^ (g >> 24)) ^ g;
@@ -76,13 +72,13 @@ string_hash (const char *xv)
 }
 
 /* Return a suitable size for a hash table, with at least SIZE slots. */
-static Elemcount
-hash_table_size (Elemcount requested_size)
+static size_t
+hash_table_size (size_t requested_size)
 {
   /* Return some prime near, but greater than or equal to, SIZE.
      Decades from the time of writing, someone will have a system large
      enough that the list below will be too short... */
-  static const Elemcount primes [] =
+  static const size_t primes [] =
   {
     19, 29, 41, 59, 79, 107, 149, 197, 263, 347, 457, 599, 787, 1031,
     1361, 1777, 2333, 3037, 3967, 5167, 6719, 8737, 11369, 14783,
@@ -92,7 +88,7 @@ hash_table_size (Elemcount requested_size)
     10445899, 13579681, 17653589, 22949669, 29834603, 38784989,
     50420551, 65546729, 85210757, 110774011, 144006217, 187208107,
     243370577, 316381771, 411296309, 534685237, 695090819, 903618083,
-    1174703521, 1527114613, 1985248999 /* , 2580823717UL, 3355070839UL */
+    1174703521, 1527114613, 1985248999, 2580823717UL, 3355070839UL
   };
   /* We've heard of binary search. */
   int low, high;
@@ -120,12 +116,12 @@ gethash (const void *key, struct hash_table *hash_table, const void **ret_value)
     {
       hentry *harray = hash_table->harray;
       hash_table_test_function test_function = hash_table->test_function;
-      Elemcount size = hash_table->size;
-      Hashcode hcode_initial =
+      hash_size_t size = hash_table->size;
+      unsigned int hcode_initial =
 	hash_table->hash_function ?
 	hash_table->hash_function (key) :
-	(Hashcode) key;
-      Elemcount hcode = (Elemcount) (hcode_initial % size);
+	(unsigned long) key;
+      unsigned int hcode = hcode_initial % size;
       hentry *e = &harray [hcode];
       const void *e_key = e->key;
 
@@ -133,8 +129,8 @@ gethash (const void *key, struct hash_table *hash_table, const void **ret_value)
 	  KEYS_DIFFER_P (e_key, key, test_function) :
 	  e->contents == NULL_ENTRY)
 	{
-	  Elemcount h2 = size - 2;
-	  Elemcount incr = (Elemcount) (1 + (hcode_initial % h2));
+	  size_t h2 = size - 2;
+	  unsigned int incr = 1 + (hcode_initial % h2);
 	  do
 	    {
 	      hcode += incr; if (hcode >= size) hcode -= size;
@@ -168,7 +164,7 @@ free_hash_table (struct hash_table *hash_table)
 }
 
 struct hash_table*
-make_hash_table (Elemcount size)
+make_hash_table (hash_size_t size)
 {
   struct hash_table *hash_table = xnew_and_zero (struct hash_table);
   hash_table->size = hash_table_size (COMFORTABLE_SIZE (size));
@@ -178,7 +174,7 @@ make_hash_table (Elemcount size)
 }
 
 struct hash_table *
-make_general_hash_table (Elemcount size,
+make_general_hash_table (hash_size_t size,
 			hash_table_hash_function hash_function,
 			hash_table_test_function test_function)
 {
@@ -189,9 +185,9 @@ make_general_hash_table (Elemcount size,
 }
 
 static void
-grow_hash_table (struct hash_table *hash_table, Elemcount new_size)
+grow_hash_table (struct hash_table *hash_table, hash_size_t new_size)
 {
-  Elemcount old_size   = hash_table->size;
+  hash_size_t old_size   = hash_table->size;
   hentry     *old_harray = hash_table->harray;
 
   hash_table->size   = hash_table_size (new_size);
@@ -210,15 +206,6 @@ grow_hash_table (struct hash_table *hash_table, Elemcount new_size)
   xfree (old_harray);
 }
 
-void
-pregrow_hash_table_if_necessary (struct hash_table *hash_table,
-				 Elemcount breathing_room)
-{
-  Elemcount comfortable_size = COMFORTABLE_SIZE (hash_table->fullness);
-  if (hash_table->size < comfortable_size - breathing_room)
-    grow_hash_table (hash_table, comfortable_size + 1);
-}
-
 void
 puthash (const void *key, void *contents, struct hash_table *hash_table)
 {
@@ -230,15 +217,15 @@ puthash (const void *key, void *contents, struct hash_table *hash_table)
   else
     {
       hash_table_test_function test_function = hash_table->test_function;
-      Elemcount size = hash_table->size;
+      hash_size_t size = hash_table->size;
       hentry *harray   = hash_table->harray;
-      Hashcode hcode_initial =
+      unsigned int hcode_initial =
 	hash_table->hash_function ?
 	hash_table->hash_function (key) :
-	(Hashcode) key;
-      Elemcount hcode = (Elemcount) (hcode_initial % size);
-      Elemcount h2 = size - 2;
-      Elemcount incr = (Elemcount) (1 + (hcode_initial % h2));
+	(unsigned long) key;
+      unsigned int hcode = hcode_initial % size;
+      size_t h2 = size - 2;
+      unsigned int incr = 1 + (hcode_initial % h2);
       const void *e_key = harray [hcode].key;
       const void *oldcontents;
 
@@ -281,7 +268,7 @@ puthash (const void *key, void *contents, struct hash_table *hash_table)
       /* only increment the fullness when we used up a new hentry */
       if (!e_key || KEYS_DIFFER_P (e_key, key, test_function))
 	{
-	  Elemcount comfortable_size = COMFORTABLE_SIZE (++(hash_table->fullness));
+	  hash_size_t comfortable_size = COMFORTABLE_SIZE (++(hash_table->fullness));
 	  if (hash_table->size < comfortable_size)
 	    grow_hash_table (hash_table, comfortable_size + 1);
 	}
@@ -289,7 +276,7 @@ puthash (const void *key, void *contents, struct hash_table *hash_table)
 }
 
 static void
-rehash (hentry *harray, struct hash_table *hash_table, Elemcount size)
+rehash (hentry *harray, struct hash_table *hash_table, hash_size_t size)
 {
   hentry *limit = harray + size;
   hentry *e;
@@ -312,12 +299,12 @@ remhash (const void *key, struct hash_table *hash_table)
     {
       hentry *harray = hash_table->harray;
       hash_table_test_function test_function = hash_table->test_function;
-      Elemcount size = hash_table->size;
-      Hashcode hcode_initial =
+      hash_size_t size = hash_table->size;
+      unsigned int hcode_initial =
 	(hash_table->hash_function) ?
 	(hash_table->hash_function (key)) :
-	((Hashcode) key);
-      Elemcount hcode = (Elemcount) (hcode_initial % size);
+	((unsigned long) key);
+      unsigned int hcode = hcode_initial % size;
       hentry *e = &harray [hcode];
       const void *e_key = e->key;
 
@@ -325,8 +312,8 @@ remhash (const void *key, struct hash_table *hash_table)
 	  KEYS_DIFFER_P (e_key, key, test_function) :
 	  e->contents == NULL_ENTRY)
 	{
-	  Elemcount h2 = size - 2;
-	  Elemcount incr = (Elemcount) (1 + (hcode_initial % h2));
+	  size_t h2 = size - 2;
+	  unsigned int incr = 1 + (hcode_initial % h2);
 	  do
 	    {
 	      hcode += incr; if (hcode >= size) hcode -= size;
