@@ -26,8 +26,6 @@
 
 /* Synched up with: Not in FSF. */
 
-/* This file Mule-ized by Ben Wing, 5-15-01. */
-
 /* There are four compile-time options.
  *
  * XTOOLKIT	This will be part of an Xt program.
@@ -60,13 +58,21 @@
  *                      work on 1.2p5 and 1.6.
  */
 
+#ifdef emacs
 #include <config.h>
 #include "lisp.h"
-
-#include "sound.h"
-
 #include "sysdep.h"
 #include "syssignal.h"
+#endif
+
+#include <stdlib.h>
+#include <stdarg.h>
+#include <string.h>
+#include <stdio.h>
+
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
 
 /* NAS <= 1.2p5 defines {BIG,LITTLE}_ENDIAN in <audio/fileutil.h>,
    conflicting with GNU libc (at least); newer versions avoid this
@@ -88,29 +94,46 @@
 
 /* NAS <= 1.2p5 <audio/fileutil.h> doesn't define the NAS_ versions */
 #ifndef NAS_LITTLE_ENDIAN
-# define NAS_LITTLE_ENDIAN LITTLE_ENDIAN
-# define NAS_BIG_ENDIAN BIG_ENDIAN
+#define NAS_LITTLE_ENDIAN LITTLE_ENDIAN
+#define NAS_BIG_ENDIAN BIG_ENDIAN
 #endif
 
-#define XTOOLKIT
-#define XTEVENTS
-#define ROBUST_PLAY
-#define CACHE_SOUNDS
+#ifdef emacs
+
+#    define XTOOLKIT
+#    define XTEVENTS
+#    define ROBUST_PLAY
+#    define CACHE_SOUNDS
 
     /*
      * For old NAS libraries, force playing to be synchronous
      * and declare the long jump point locally.
      */
 
-#if defined (NAS_NO_ERROR_JUMP)
-# undef XTEVENTS
-# include <setjmp.h>
-jmp_buf AuXtErrorJump;
-#endif
+#    if defined (NAS_NO_ERROR_JUMP)
+
+#	undef XTEVENTS
+
+#	include <setjmp.h>
+	jmp_buf AuXtErrorJump;
+#    endif
+
+     /* The GETTEXT is correct. --ben */
+#    define warn(str) warn_when_safe (Qnas, Qwarning, "nas: %s ", GETTEXT (str))
+
+#    define play_sound_file nas_play_sound_file
+#    define play_sound_data nas_play_sound_data
+#    define wait_for_sounds nas_wait_for_sounds
+#    define init_play       nas_init_play
+#    define close_down_play nas_close_down_play
+
+#else /* !emacs */
+#    define warn(str) fprintf (stderr, "%s\n", (str))
+#endif /* emacs */
 
 #ifdef XTOOLKIT
-# include <X11/Intrinsic.h>
-# include <audio/Xtutil.h>
+#    include <X11/Intrinsic.h>
+#    include <audio/Xtutil.h>
 #endif
 
 #if defined (ROBUST_PLAY)
@@ -121,7 +144,7 @@ SIGTYPE sigpipe_handle (int signo);
 
 extern Lisp_Object Vsynchronous_sounds;
 
-static Sound SoundOpenDataForReading (UChar_Binary *data, int length);
+static Sound SoundOpenDataForReading (unsigned char *data, int length);
 
 static AuServer       *aud;
 
@@ -133,31 +156,31 @@ static int sounds_in_play;
 static Display *aud_server;
 static XtInputId input_id;
 #else
-static Extbyte *aud_server;
+static char *aud_server;
 #endif /* XTOOLKIT */
 
-Extbyte *
-nas_init_play (
+char *
+init_play (
 #ifdef XTOOLKIT
 	   Display *display
 #else
-	   Extbyte *server
+	   char *server
 #endif
 	   );
-Extbyte *
-nas_init_play (
+char *
+init_play (
 #ifdef XTOOLKIT
 	   Display *display
 #else
-	   Extbyte *server
+	   char *server
 #endif
 	   )
 {
-  Extbyte *err_message;
+  char *err_message;
   SIGTYPE (*old_sigpipe) (int);
 
 #ifdef XTOOLKIT
-  Extbyte * server = DisplayString (display);
+  char * server = DisplayString (display);
   XtAppContext app_context = XtDisplayToApplicationContext (display);
 
   aud_server = display;
@@ -167,11 +190,13 @@ nas_init_play (
 #endif
 
 #ifdef ROBUST_PLAY
-  old_sigpipe = EMACS_SIGNAL (SIGPIPE, sigpipe_handle);
+  old_sigpipe = signal (SIGPIPE, sigpipe_handle);
   if (setjmp (AuXtErrorJump))
     {
-      EMACS_SIGNAL (SIGPIPE, old_sigpipe);
+      signal (SIGPIPE, old_sigpipe);
+#ifdef emacs
       start_interrupts ();
+#endif  
       return "error in NAS";
     }
 #endif
@@ -181,13 +206,17 @@ nas_init_play (
   AuDefaultErrorHandler = CatchErrorAndJump;
 #endif
 
+#ifdef emacs
   stop_interrupts ();
+#endif  
   aud = AuOpenServer (server, 0, NULL, 0, NULL, &err_message);
+#ifdef emacs
   start_interrupts ();
+#endif  
   if (!aud)
     {
 #ifdef ROBUST_PLAY
-      EMACS_SIGNAL (SIGPIPE, old_sigpipe);
+      signal (SIGPIPE, old_sigpipe);
 #endif
       if (err_message == NULL)
 	return "Can't connect to audio server";
@@ -214,7 +243,7 @@ nas_init_play (
 #endif
 
 #ifdef ROBUST_PLAY
-  EMACS_SIGNAL (SIGPIPE, old_sigpipe);
+  signal (SIGPIPE, old_sigpipe);
 #endif
 
   sounds_in_play = 0;
@@ -223,11 +252,11 @@ nas_init_play (
 }
 
 static void
-nas_close_down_play (void)
+close_down_play (void)
 
 {
   AuCloseServer (aud);
-  sound_warn ("disconnected from audio server");
+  warn ("disconnected from audio server");
 }
 
  /********************************************************************\
@@ -259,7 +288,7 @@ doneCB (AuServer       *auserver,
 static void
 do_caching_play (Sound s,
 		 int volume,
-		 UChar_Binary *buf)
+		 unsigned char *buf)
 
 {
   AuBucketAttributes *list, b;
@@ -282,7 +311,7 @@ do_caching_play (Sound s,
 	      return;
 	    }
 
-	  if (SoundReadFile ((Extbyte *) my_buf, SoundNumBytes (s), s) != SoundNumBytes (s))
+	  if (SoundReadFile ((char *) my_buf, SoundNumBytes (s), s) != SoundNumBytes (s))
 	    {
 	      free (my_buf);
 	      return;
@@ -321,9 +350,9 @@ do_caching_play (Sound s,
 #endif /* CACHE_SOUNDS */
 
 
-void nas_wait_for_sounds (void);
+void wait_for_sounds (void);
 void 
-nas_wait_for_sounds (void)
+wait_for_sounds (void)
 
 {
   AuEvent         ev;
@@ -335,46 +364,45 @@ nas_wait_for_sounds (void)
     }
 }
 
-int nas_play_sound_file (Extbyte *sound_file, int volume);
+int play_sound_file (char *sound_file, int volume);
 int
-nas_play_sound_file (Extbyte *sound_file,
-		     int volume)
+play_sound_file (char *sound_file,
+		 int volume)
 {
   SIGTYPE (*old_sigpipe) (int);
 
 #ifdef ROBUST_PLAY
-  old_sigpipe = EMACS_SIGNAL (SIGPIPE, sigpipe_handle);
+  old_sigpipe=signal (SIGPIPE, sigpipe_handle);
   if (setjmp (AuXtErrorJump))
     {
-      EMACS_SIGNAL (SIGPIPE, old_sigpipe);
+      signal (SIGPIPE, old_sigpipe);
       return 0;
     }
 #endif
 
-  if (aud==NULL)
-    {
-      if (aud_server != NULL)
-	{
-	  Extbyte *m;
-	  /* attempt to reconect */
-	  if ((m = nas_init_play (aud_server)) != NULL)
-	    {
-	      
+  if (aud==NULL) {
+    if (aud_server != NULL)
+      {
+	char *m;
+	/* attempt to reconect */
+	if ((m=init_play (aud_server))!= NULL)
+	  {
+
 #ifdef ROBUST_PLAY
-	      EMACS_SIGNAL (SIGPIPE, old_sigpipe);
+	    signal (SIGPIPE, old_sigpipe);
 #endif
-	      return 0;
-	    }
-	}
-      else
-	{
-	  sound_warn ("Attempt to play with no audio init\n");
+	    return 0;
+	  }
+      }
+    else
+      {
+	warn ("Attempt to play with no audio init\n");
 #ifdef ROBUST_PLAY
-	  EMACS_SIGNAL (SIGPIPE, old_sigpipe);
+	signal (SIGPIPE, old_sigpipe);
 #endif
-	  return 0;
-	}
-    }
+	return 0;
+      }
+  }
 
 #ifndef CACHE_SOUNDS
   sounds_in_play++;
@@ -396,7 +424,7 @@ nas_play_sound_file (Extbyte *sound_file,
     if ((s = SoundOpenFileForReading (sound_file))==NULL)
       {
 #ifdef ROBUST_PLAY
-	EMACS_SIGNAL (SIGPIPE, old_sigpipe);
+	signal (SIGPIPE, old_sigpipe);
 #endif
 	return 0;
       }
@@ -414,22 +442,26 @@ nas_play_sound_file (Extbyte *sound_file,
 #endif /* CACHE_SOUNDS */
 
 #ifndef XTEVENTS
-  nas_wait_for_sounds ();
+  wait_for_sounds ();
 #else
   if (!NILP (Vsynchronous_sounds))
-    nas_wait_for_sounds ();
+    {
+      wait_for_sounds ();
+    }
 #endif
 
 #ifdef ROBUST_PLAY
-  EMACS_SIGNAL (SIGPIPE, old_sigpipe);
+  signal (SIGPIPE, old_sigpipe);
 #endif
 
   return 1;
 }
 
-int nas_play_sound_data (UChar_Binary *data, int length, int volume);
+int play_sound_data (unsigned char *data, int length, int volume);
 int
-nas_play_sound_data (UChar_Binary *data, int length, int volume)
+play_sound_data (unsigned char *data,
+		 int length, 
+		 int volume)
 {
   Sound s;
   int offset;
@@ -440,10 +472,10 @@ nas_play_sound_data (UChar_Binary *data, int length, int volume)
 #endif
 
 #ifdef ROBUST_PLAY
-  old_sigpipe = EMACS_SIGNAL (SIGPIPE, sigpipe_handle);
-  if (setjmp (AuXtErrorJump) != 0)
+  old_sigpipe = signal (SIGPIPE, sigpipe_handle);
+  if (setjmp (AuXtErrorJump) !=0)
     {
-      EMACS_SIGNAL (SIGPIPE, old_sigpipe);
+      signal (SIGPIPE, old_sigpipe);
       return 0;
     }
 #endif
@@ -452,21 +484,21 @@ nas_play_sound_data (UChar_Binary *data, int length, int volume)
   if (aud == NULL) {
     if (aud_server != NULL)
       {
-	Extbyte *m;
+	char *m;
 	/* attempt to reconect */
-	if ((m = nas_init_play (aud_server)) != NULL)
+	if ((m = init_play (aud_server)) != NULL)
 	  {
 #ifdef ROBUST_PLAY
-	    EMACS_SIGNAL (SIGPIPE, old_sigpipe);
+	    signal (SIGPIPE, old_sigpipe);
 #endif
 	    return 0;
 	  }
       }
     else
       {
-	sound_warn ("Attempt to play with no audio init\n");
+	warn ("Attempt to play with no audio init\n");
 #ifdef ROBUST_PLAY
-	EMACS_SIGNAL (SIGPIPE, old_sigpipe);
+	signal (SIGPIPE, old_sigpipe);
 #endif
 	return 0;
       }
@@ -474,9 +506,9 @@ nas_play_sound_data (UChar_Binary *data, int length, int volume)
 
   if ((s=SoundOpenDataForReading (data, length))==NULL)
     {
-      sound_warn ("unknown sound type");
+      warn ("unknown sound type");
 #ifdef ROBUST_PLAY
-      EMACS_SIGNAL (SIGPIPE, old_sigpipe);
+      signal (SIGPIPE, old_sigpipe);
 #endif
       return 0;
     }
@@ -492,10 +524,10 @@ nas_play_sound_data (UChar_Binary *data, int length, int volume)
     }
   else
     {
-      sound_warn ("only understand snd and wave files at the moment");
+      warn ("only understand snd and wave files at the moment");
       SoundCloseFile (s);
 #ifdef ROBUST_PLAY
-      EMACS_SIGNAL (SIGPIPE, old_sigpipe);
+      signal (SIGPIPE, old_sigpipe);
 #endif
       return 0;
     }
@@ -515,21 +547,25 @@ nas_play_sound_data (UChar_Binary *data, int length, int volume)
 #else
   /* Cache the sounds in buckets on the server */
 
-  do_caching_play (s, volume, data+offset);
+  {
+    do_caching_play (s, volume, data+offset);
+  }
 #endif /* CACHE_SOUNDS */
 
 
 #ifndef XTEVENTS
-  nas_wait_for_sounds ();
+  wait_for_sounds ();
 #else
   if (!NILP (Vsynchronous_sounds))
-    nas_wait_for_sounds ();
+    {
+      wait_for_sounds ();
+    }
 #endif
 
   SoundCloseFile (s); 
 
 #ifdef ROBUST_PLAY
-  EMACS_SIGNAL (SIGPIPE, old_sigpipe);
+  signal (SIGPIPE, old_sigpipe);
 #endif
 
   return 1;
@@ -549,13 +585,15 @@ static AuBool
 CatchIoErrorAndJump (AuServer *old_aud)
 {
   if (old_aud)
-    sound_warn ("Audio Server connection broken"); 
+    warn ("Audio Server connection broken"); 
   else
-    sound_warn ("Audio Server connection broken because of signal");
+    warn ("Audio Server connection broken because of signal");
 
 #ifdef XTEVENTS
 #ifdef XTOOLKIT
-  AuXtAppRemoveAudioHandler (aud, input_id); 
+  {
+    AuXtAppRemoveAudioHandler (aud, input_id); 
+  }
 #endif
 
   if (aud)
@@ -605,14 +643,14 @@ CatchErrorAndJump (AuServer *old_aud,
 
 /* Create a name from the sound. */
 
-static Extbyte *
-NameFromData (const Char_Binary *buf,
+static char *
+NameFromData (const char *buf,
 	      int len)
 
 {
-  Extbyte name[9];
+  char name[9];
   int i;
-  Extbyte *s;
+  char *s;
 
   buf+=len/2;
   len -= len/2;
@@ -635,11 +673,11 @@ NameFromData (const Char_Binary *buf,
 
   if (i==8)
     {
-      strcpy (s = (Extbyte *) malloc (10), name);
+      strcpy (s = (char *) malloc (10), name);
     }
   else 
     {
-      strcpy (s = (Extbyte *) malloc (15), "short sound");
+      strcpy (s = (char *) malloc (15), "short sound");
     }
 
   return s;
@@ -650,7 +688,7 @@ NameFromData (const Char_Binary *buf,
  */
 
 static SndInfo *
-SndOpenDataForReading (const Char_Binary *data,
+SndOpenDataForReading (const char *data,
 		       int length)
 
 {
@@ -667,7 +705,7 @@ SndOpenDataForReading (const Char_Binary *data,
 
   if (NAS_LITTLE_ENDIAN)
     {
-      Char_Binary            n;
+      char            n;
     
       swapl (&si->h.magic, n);
       swapl (&si->h.dataOffset, n);
@@ -687,7 +725,7 @@ SndOpenDataForReading (const Char_Binary *data,
 
   if (size)
     {
-      if (!(si->comment = (Extbyte *) malloc (size + 1)))
+      if (!(si->comment = (char *) malloc (size + 1)))
 	{
 	  free (si);
 	  return NULL;
@@ -713,10 +751,10 @@ SndOpenDataForReading (const Char_Binary *data,
 /* Stuff taken from wave.c from NAS.  Just like snd files, NAS can't
    read wave data from memory, so these functions do that for us. */
 
-#define Err()		{ return NULL; }
+#define Err()		{ free (wi); return NULL; }
 #define readFourcc(_f)	dread(_f, sizeof(RIFF_FOURCC), 1)
 #define cmpID(_x, _y)							      \
-    strncmp((Char_Binary *) (_x), (Char_Binary *) (_y), sizeof(RIFF_FOURCC))
+    strncmp((char *) (_x), (char *) (_y), sizeof(RIFF_FOURCC))
 #define PAD2(_x)	(((_x) + 1) & ~1)
 
 /* These functions here are for faking file I/O from buffer. */
@@ -745,7 +783,7 @@ dread (void* buf, size_t size, size_t nitems)
   
   if (file_posn + nread <= file_len)
     {
-      memcpy(buf, (Char_Binary *) file_data + file_posn, size * nitems);
+      memcpy(buf, (char *) file_data + file_posn, size * nitems);
       file_posn += nread;
       return nitems;
     }
@@ -760,7 +798,7 @@ static int
 dgetc (void)
 {
   if (file_posn < file_len)
-    return ((Char_Binary *)file_data)[file_posn++];
+    return ((char *)file_data)[file_posn++];
   else
     return -1;
 }
@@ -791,177 +829,177 @@ dtell (void)
 static unsigned short
 DataReadS (int swapit)
 {
-  unsigned short us;
+    unsigned short us;
 
-  dread(&us, 2, 1);
-  if (swapit)
-    us = FileSwapS(us);
-  return us;
+    dread(&us, 2, 1);
+    if (swapit)
+	us = FileSwapS(us);
+    return us;
 }
 
 static AuUint32
 DataReadL (int swapit)
 {
-  AuUint32 ul;
+    AuUint32 ul;
 
-  dread(&ul, 4, 1);
-  if (swapit)
-    ul = FileSwapL(ul);
-  return ul;
+    dread(&ul, 4, 1);
+    if (swapit)
+	ul = FileSwapL(ul);
+    return ul;
 }
 
 static int
 readChunk (RiffChunk *c)
 {
-  int             status;
-  Char_Binary            n;
+    int             status;
+    char            n;
 
-  if ((status = dread(c, sizeof(RiffChunk), 1)))
-    if (NAS_BIG_ENDIAN)
-      swapl(&c->ckSize, n);
+    if ((status = dread(c, sizeof(RiffChunk), 1)))
+	if (NAS_BIG_ENDIAN)
+	    swapl(&c->ckSize, n);
 
-  return status;
+    return status;
 }
 
 /* A very straight-forward translation of WaveOpenFileForReading to
    read the wave data from a buffer in memory. */
 
 static WaveInfo *
-WaveOpenDataForReading (const Char_Binary *data,
+WaveOpenDataForReading (const char *data,
 			int length)
 {
-  RiffChunk       ck;
-  RIFF_FOURCC     fourcc;
-  AuInt32            fileSize;
-  WaveInfo       *wi;
+    RiffChunk       ck;
+    RIFF_FOURCC     fourcc;
+    AuInt32            fileSize;
+    WaveInfo       *wi;
 
     
-  if (!(wi = (WaveInfo *) malloc(sizeof(WaveInfo))))
-    return NULL;
+    if (!(wi = (WaveInfo *) malloc(sizeof(WaveInfo))))
+	return NULL;
 
-  wi->comment = NULL;
-  wi->dataOffset = wi->format = wi->writing = 0;
+    wi->comment = NULL;
+    wi->dataOffset = wi->format = wi->writing = 0;
 
-  dopen(data, length);
+    dopen(data, length);
     
-  if (!readChunk(&ck) ||
-      cmpID(&ck.ckID, RIFF_RiffID) ||
-      !readFourcc(&fourcc) ||
-      cmpID(&fourcc, RIFF_WaveID))
-    Err();
-
-  fileSize = PAD2(ck.ckSize) - sizeof(RIFF_FOURCC);
-
-  while (fileSize >= (AuInt32) sizeof(RiffChunk))
-    {
-      if (!readChunk(&ck))
+    if (!readChunk(&ck) ||
+	cmpID(&ck.ckID, RIFF_RiffID) ||
+	!readFourcc(&fourcc) ||
+	cmpID(&fourcc, RIFF_WaveID))
 	Err();
 
-      fileSize -= sizeof(RiffChunk) + PAD2(ck.ckSize);
+    fileSize = PAD2(ck.ckSize) - sizeof(RIFF_FOURCC);
 
-      /* LIST chunk */
-      if (!cmpID(&ck.ckID, RIFF_ListID))
-	{
-	  if (!readFourcc(&fourcc))
+    while (fileSize >= (AuInt32) sizeof(RiffChunk))
+    {
+	if (!readChunk(&ck))
 	    Err();
 
-	  /* INFO chunk */
-	  if (!cmpID(&fourcc, RIFF_ListInfoID))
+	fileSize -= sizeof(RiffChunk) + PAD2(ck.ckSize);
+
+	/* LIST chunk */
+	if (!cmpID(&ck.ckID, RIFF_ListID))
+	{
+	    if (!readFourcc(&fourcc))
+		Err();
+
+	    /* INFO chunk */
+	    if (!cmpID(&fourcc, RIFF_ListInfoID))
 	    {
-	      ck.ckSize -= sizeof(RIFF_FOURCC);
+		ck.ckSize -= sizeof(RIFF_FOURCC);
 
-	      while (ck.ckSize)
+		while (ck.ckSize)
 		{
-		  RiffChunk       c;
+		    RiffChunk       c;
 
-		  if (!readChunk(&c))
-		    Err();
-
-		  /* ICMT chunk */
-		  if (!cmpID(&c.ckID, RIFF_InfoIcmtID))
-		    {
-		      if (!(wi->comment = (Extbyte *) malloc(c.ckSize)) ||
-			  !dread(wi->comment, c.ckSize, 1))
+		    if (!readChunk(&c))
 			Err();
 
-		      if (c.ckSize & 1)
-			dgetc(); /* eat the pad byte */
-		    }
-		  else
-		    /* skip unknown chunk */
-		    dseek(PAD2(c.ckSize), 1);
+		    /* ICMT chunk */
+		    if (!cmpID(&c.ckID, RIFF_InfoIcmtID))
+		    {
+			if (!(wi->comment = (char *) malloc(c.ckSize)) ||
+			    !dread(wi->comment, c.ckSize, 1))
+			    Err();
 
-		  ck.ckSize -= sizeof(RiffChunk) + PAD2(c.ckSize);
+			if (c.ckSize & 1)
+			    dgetc();	/* eat the pad byte */
+		    }
+		    else
+			/* skip unknown chunk */
+			dseek(PAD2(c.ckSize), 1);
+
+		    ck.ckSize -= sizeof(RiffChunk) + PAD2(c.ckSize);
 		}
 	    }
-	  else
-	    /* skip unknown chunk */
-	    dseek(PAD2(ck.ckSize) - sizeof(RIFF_FOURCC), 1);
+	    else
+		/* skip unknown chunk */
+		dseek(PAD2(ck.ckSize) - sizeof(RIFF_FOURCC), 1);
 	}
-      /* wave format chunk */
-      else if (!cmpID(&ck.ckID, RIFF_WaveFmtID) && !wi->format)
+	/* wave format chunk */
+	else if (!cmpID(&ck.ckID, RIFF_WaveFmtID) && !wi->format)
 	{
-	  AuInt32            dummy;
+	    AuInt32            dummy;
 
-	  wi->format = DataReadS(NAS_BIG_ENDIAN);
-	  wi->channels = DataReadS(NAS_BIG_ENDIAN);
-	  wi->sampleRate = DataReadL(NAS_BIG_ENDIAN);
+	    wi->format = DataReadS(NAS_BIG_ENDIAN);
+	    wi->channels = DataReadS(NAS_BIG_ENDIAN);
+	    wi->sampleRate = DataReadL(NAS_BIG_ENDIAN);
 
-	  /* we don't care about the next two fields */
-	  dummy = DataReadL(NAS_BIG_ENDIAN);
-	  dummy = DataReadS(NAS_BIG_ENDIAN);
+	    /* we don't care about the next two fields */
+	    dummy = DataReadL(NAS_BIG_ENDIAN);
+	    dummy = DataReadS(NAS_BIG_ENDIAN);
 
-	  if (wi->format != RIFF_WAVE_FORMAT_PCM)
-	    Err();
+	    if (wi->format != RIFF_WAVE_FORMAT_PCM)
+		Err();
 
-	  wi->bitsPerSample = DataReadS(NAS_BIG_ENDIAN);
+	    wi->bitsPerSample = DataReadS(NAS_BIG_ENDIAN);
 
-	  /* skip any other format specific fields */
-	  dseek(PAD2(ck.ckSize - 16), 1);
+	    /* skip any other format specific fields */
+	    dseek(PAD2(ck.ckSize - 16), 1);
 	}
-      /* wave data chunk */
-      else if (!cmpID(&ck.ckID, RIFF_WaveDataID) && !wi->dataOffset)
+	/* wave data chunk */
+	else if (!cmpID(&ck.ckID, RIFF_WaveDataID) && !wi->dataOffset)
 	{
-	  long endOfFile;
+	    long endOfFile;
 
-	  wi->dataOffset = dtell();
-	  wi->dataSize = ck.ckSize;
-	  dseek(0, 2);
-	  endOfFile = dtell();
+	    wi->dataOffset = dtell();
+	    wi->dataSize = ck.ckSize;
+	    dseek(0, 2);
+	    endOfFile = dtell();
 
-	  /* seek past the data */
-	  if (dseek(wi->dataOffset + PAD2(ck.ckSize), 0) ||
-	      dtell() > endOfFile)
+	    /* seek past the data */
+	    if (dseek(wi->dataOffset + PAD2(ck.ckSize), 0) ||
+		dtell() > endOfFile)
 	    {
-	      /* the seek failed, assume the size is bogus */
-	      dseek(0, 2);
-	      wi->dataSize = dtell() - wi->dataOffset;
+		/* the seek failed, assume the size is bogus */
+		dseek(0, 2);
+		wi->dataSize = dtell() - wi->dataOffset;
 	    }
 
-	  wi->dataOffset -= sizeof(long);
+	    wi->dataOffset -= sizeof(long);
 	}
-      else
-	/* skip unknown chunk */
-	dseek(PAD2(ck.ckSize), 1);
+	else
+	    /* skip unknown chunk */
+	    dseek(PAD2(ck.ckSize), 1);
     }
 
-  if (!wi->dataOffset)
-    Err();
+    if (!wi->dataOffset)
+	Err();
 
-  wi->numSamples = wi->dataSize / wi->channels / (wi->bitsPerSample >> 3);
+    wi->numSamples = wi->dataSize / wi->channels / (wi->bitsPerSample >> 3);
 
-  if (!wi->comment)
-    wi->comment = NameFromData (data + wi->dataOffset,
-				length - wi->dataOffset);
+    if (!wi->comment)
+       wi->comment = NameFromData (data + wi->dataOffset,
+                                   length - wi->dataOffset);
 
-  wi->fp = NULL;
+    wi->fp = NULL;
     
-  return wi;
+    return wi;
 }
 
 
 static Sound
-SoundOpenDataForReading (UChar_Binary *data,
+SoundOpenDataForReading (unsigned char *data,
 			 int length)
 
 {
@@ -973,7 +1011,7 @@ SoundOpenDataForReading (UChar_Binary *data,
   if (!(s = (Sound) malloc (sizeof (SoundRec))))
     return NULL;
 
-  if ((s->formatInfo = SndOpenDataForReading ((Char_Binary *) data, length)) != NULL)
+  if ((s->formatInfo = SndOpenDataForReading ((char *) data, length)) != NULL)
     {
 #if (AudioLibraryVersionMajor >= 2 ) && (AudioLibraryVersionMinor >= 3)
       if ((toProc = SoundFileGetProc(SoundFileFormatSnd, 
@@ -994,7 +1032,7 @@ SoundOpenDataForReading (UChar_Binary *data,
 	  return NULL;
 	}
     }
-  else if ((s->formatInfo = WaveOpenDataForReading ((Char_Binary *) data, length)) != NULL)
+  else if ((s->formatInfo = WaveOpenDataForReading ((char *) data, length)) != NULL)
     {
 #if (AudioLibraryVersionMajor >= 2 ) && (AudioLibraryVersionMinor >= 3)
       if ((toProc = SoundFileGetProc(SoundFileFormatWave, 
