@@ -1,6 +1,6 @@
 /* Synchronize redisplay structures and output changes.
    Copyright (C) 1994, 1995 Board of Trustees, University of Illinois.
-   Copyright (C) 1995, 1996, 2002, 2003 Ben Wing.
+   Copyright (C) 1995, 1996 Ben Wing.
    Copyright (C) 1996 Chuck Thompson.
    Copyright (C) 1999, 2002 Andy Piper.
 
@@ -35,8 +35,8 @@ Boston, MA 02111-1307, USA.  */
 
 #include "buffer.h"
 #include "window.h"
-#include "frame-impl.h"
-#include "device-impl.h"
+#include "frame.h"
+#include "device.h"
 #include "glyphs.h"
 #include "redisplay.h"
 #include "faces.h"
@@ -59,8 +59,6 @@ static void redisplay_clear_clipped_region (Lisp_Object locale, face_index finde
 					    struct display_box* dest,
 					    struct display_glyph_area* glyphsrc,
 					    int fullheight_p, Lisp_Object);
-static void redisplay_redraw_exposed_windows (Lisp_Object window, int x,
-					      int y, int width, int height);
 
 /*****************************************************************************
  sync_rune_structs
@@ -109,16 +107,16 @@ sync_display_line_structs (struct window *w, int line, int do_blocks,
 			   display_line_dynarr *cdla,
 			   display_line_dynarr *ddla)
 {
+  int cdla_len = Dynarr_length (cdla);
+
   struct display_line dl, *clp, *dlp;
   int db_elt;
-  int local = 0;
 
   dlp = Dynarr_atp (ddla, line);
   if (line >= Dynarr_largest (cdla))
     {
       clp = &dl;
       clp->display_blocks = Dynarr_new (display_block);
-      local = 1;
     }
   else
     {
@@ -145,42 +143,40 @@ sync_display_line_structs (struct window *w, int line, int do_blocks,
     clp->right_glyphs = 0;
   }
 
-  if (do_blocks || line < Dynarr_length (cdla))
+  if (!do_blocks && line >= cdla_len)
     {
-      for (db_elt = 0; db_elt < Dynarr_length (dlp->display_blocks); db_elt++)
+      Dynarr_add (cdla, *clp);
+      return;
+    }
+
+  for (db_elt = 0; db_elt < Dynarr_length (dlp->display_blocks); db_elt++)
+    {
+      struct display_block db, *cdb;
+      struct display_block *ddb = Dynarr_atp (dlp->display_blocks, db_elt);
+
+      if (db_elt >= Dynarr_largest (clp->display_blocks))
 	{
-	  struct display_block db, *cdb;
-	  struct display_block *ddb = Dynarr_atp (dlp->display_blocks, db_elt);
-
-	  if (db_elt >= Dynarr_largest (clp->display_blocks))
-	    {
-	      cdb = &db;
-	      memcpy (cdb, ddb, sizeof (struct display_block));
-	      cdb->runes = Dynarr_new (rune);
-	      Dynarr_add (clp->display_blocks, *cdb);
-	    }
-	  else
-	    {
-	      rune_dynarr *tr;
-
-	      cdb = Dynarr_atp (clp->display_blocks, db_elt);
-	      tr = cdb->runes;
-	      memcpy (cdb, ddb, sizeof (struct display_block));
-	      cdb->runes = tr;
-	      Dynarr_increment (clp->display_blocks);
-	    }
-
-	  sync_rune_structs (w, cdb->runes, ddb->runes);
+	  cdb = &db;
+	  memcpy (cdb, ddb, sizeof (struct display_block));
+	  cdb->runes = Dynarr_new (rune);
+	  Dynarr_add (clp->display_blocks, *cdb);
 	}
+      else
+	{
+	  rune_dynarr *tr;
+
+	  cdb = Dynarr_atp (clp->display_blocks, db_elt);
+	  tr = cdb->runes;
+	  memcpy (cdb, ddb, sizeof (struct display_block));
+	  cdb->runes = tr;
+	  Dynarr_increment (clp->display_blocks);
+	}
+
+      sync_rune_structs (w, cdb->runes, ddb->runes);
     }
 
-  if (local)
+  if (line >= cdla_len)
     Dynarr_add (cdla, *clp);
-  else if (line >= Dynarr_length (cdla))
-    {
-      assert (line == Dynarr_length (cdla));
-      Dynarr_increment (cdla);
-    }
 }
 
 /*****************************************************************************
@@ -192,7 +188,7 @@ sync_display_line_structs (struct window *w, int line, int do_blocks,
 static int
 compare_runes (struct window *w, struct rune *crb, struct rune *drb)
 {
-  /* Do not compare the values of charbpos and endpos.  They do not
+  /* Do not compare the values of bufpos and endpos.  They do not
      affect the display characteristics. */
 
   /* Note: (hanoi 6) spends 95% of its time in redisplay, and about
@@ -255,9 +251,9 @@ compare_runes (struct window *w, struct rune *crb, struct rune *drb)
          then clear in redisplay_output_layout (). */
       Lisp_Object window, image;
       Lisp_Image_Instance* ii;
-      window = wrap_window (w);
+      XSETWINDOW (window, w);
       image = glyph_image_instance (crb->object.dglyph.glyph,
-				    window, ERROR_ME_DEBUG_WARN, 1);
+				    window, ERROR_ME_NOT, 1);
 
       if (!IMAGE_INSTANCEP (image))
 	return 0;
@@ -307,86 +303,6 @@ compare_runes (struct window *w, struct rune *crb, struct rune *drb)
     return 1;
 }
 
-#if 0
-void
-compare_runes_2 (struct window *w, struct rune *crb, struct rune *drb)
-{
-  if (crb->type == DGLYPH)
-    {
-      if (!EQ (crb->object.dglyph.glyph, drb->object.dglyph.glyph) ||
-	  !EQ (crb->object.dglyph.extent, drb->object.dglyph.extent) ||
-	  crb->object.dglyph.xoffset != drb->object.dglyph.xoffset ||
-	  crb->object.dglyph.yoffset != drb->object.dglyph.yoffset ||
-	  crb->object.dglyph.ascent != drb->object.dglyph.ascent ||
-	  crb->object.dglyph.descent != drb->object.dglyph.descent)
-	return 0;
-      /* Only check dirtiness if we know something has changed. */
-      else if (XGLYPH_DIRTYP (crb->object.dglyph.glyph) ||
-	       crb->findex != drb->findex)
-	{
-	  /* We need some way of telling redisplay_output_layout () that the
-	     only reason we are outputting it is because something has
-	     changed internally. That way we can optimize whether we need
-	     to clear the layout first and also only output the components
-	     that have changed. The image_instance dirty flag and
-	     display_hash are no good to us because these will invariably
-	     have been set anyway if the layout has changed. So it looks
-	     like we need yet another change flag that we can set here and
-	     then clear in redisplay_output_layout (). */
-	  Lisp_Object window, image;
-	  Lisp_Image_Instance* ii;
-	  window = wrap_window (w);
-	  image = glyph_image_instance (crb->object.dglyph.glyph,
-					window, crb->object.dglyph.matchspec,
-					ERROR_ME_DEBUG_WARN, 1);
-	  
-	  if (!IMAGE_INSTANCEP (image))
-	    return 0;
-	  ii = XIMAGE_INSTANCE (image);
-	  
-	  if (TEXT_IMAGE_INSTANCEP (image) &&
-	      (crb->findex != drb->findex ||
-	       WINDOW_FACE_CACHEL_DIRTY (w, drb->findex)))
-	    return 0;
-	  
-	  /* It is quite common for the two glyphs to be EQ since in many
-	     cases they will actually be the same object. This does not
-	     mean, however, that nothing has changed. We therefore need to
-	     check the current hash of the glyph against the last recorded
-	     display hash and the pending display items. See
-	     update_widget () ^^#### which function?. */
-	  if (image_instance_changed (image) ||
-	      crb->findex != drb->findex ||
-	      WINDOW_FACE_CACHEL_DIRTY (w, drb->findex))
-	    {
-	      /* Now we are going to re-output the glyph, but since
-		 this is for some internal reason not related to geometry
-		 changes, send a hint to the output routines that they can
-		 take some short cuts. This is most useful for
-		 layouts. This flag should get reset by the output
-		 routines.
-		 
-		 #### It is possible for us to get here when the
-		 face_cachel is dirty. I do not know what the implications
-		 of this are.*/
-	      IMAGE_INSTANCE_OPTIMIZE_OUTPUT (ii) = 1;
-	      return 0;
-	    }
-	  else
-	    return 1;
-	}
-      else if (crb->findex != drb->findex ||
-	       WINDOW_FACE_CACHEL_DIRTY (w, drb->findex))
-	return 0;
-      else
-	return 1;
-    }
-  else return !(memcmp (crb, drb, sizeof (*crb)) ||
-		WINDOW_FACE_CACHEL_DIRTY (w, drb->findex));
-}
-#endif
-
-
 /*****************************************************************************
  get_next_display_block
 
@@ -418,7 +334,7 @@ get_next_display_block (layout_bounds bounds, display_block_dynarr *dba,
       else if (start_pos <= bounds.right_out)
 	*next_start = bounds.right_out;
       else
-	abort ();
+	ABORT ();
     }
 
   for (block = 0; block < Dynarr_length (dba); block++)
@@ -461,9 +377,9 @@ get_cursor_size_and_location (struct window *w, struct display_block *db,
   int defheight, defwidth;
 
   if (Dynarr_length (db->runes) <= cursor_location)
-    abort ();
+    ABORT ();
 
-  window = wrap_window (w);
+  XSETWINDOW (window, w);
 
   rb = Dynarr_atp (db->runes, cursor_location);
   *cursor_start = rb->xpos;
@@ -623,8 +539,9 @@ static void
 clear_left_border (struct window *w, int y, int height)
 {
   struct frame *f = XFRAME (w->frame);
-  Lisp_Object window = wrap_window (w);
+  Lisp_Object window;
 
+  XSETWINDOW (window, w);
   redisplay_clear_region (window, DEFAULT_INDEX,
 		FRAME_LEFT_BORDER_START (f), y,
 		FRAME_BORDER_WIDTH (f), height);
@@ -639,8 +556,9 @@ static void
 clear_right_border (struct window *w, int y, int height)
 {
   struct frame *f = XFRAME (w->frame);
-  Lisp_Object window = wrap_window (w);
+  Lisp_Object window;
 
+  XSETWINDOW (window, w);
   redisplay_clear_region (window, DEFAULT_INDEX,
 		FRAME_RIGHT_BORDER_START (f),
 		y, FRAME_BORDER_WIDTH (f), height);
@@ -810,8 +728,9 @@ output_display_line (struct window *w, display_line_dynarr *cdla,
 
 		  if (findex != (face_index) -1)
 		    {
-		      Lisp_Object window = wrap_window (w);
+		      Lisp_Object window;
 
+		      XSETWINDOW (window, w);
 
 		      /* Clear the empty area. */
 		      redisplay_clear_region (window, findex, x, y, width, height);
@@ -930,11 +849,11 @@ output_display_line (struct window *w, display_line_dynarr *cdla,
  boolean indicating success or failure.
  ****************************************************************************/
 
-#define ADJ_CHARPOS (rb->charpos + dl->offset)
+#define ADJ_BUFPOS (rb->bufpos + dl->offset)
 #define ADJ_ENDPOS (rb->endpos + dl->offset)
 
 int
-redisplay_move_cursor (struct window *w, Charbpos new_point, int no_output_end)
+redisplay_move_cursor (struct window *w, Bufpos new_point, int no_output_end)
 {
   struct frame *f = XFRAME (w->frame);
   struct device *d = XDEVICE (f->device);
@@ -970,13 +889,13 @@ redisplay_move_cursor (struct window *w, Charbpos new_point, int no_output_end)
 
   if (rb->cursor_type == CURSOR_OFF)
     return 0;
-  else if (ADJ_CHARPOS == new_point
-	   || (ADJ_ENDPOS && (new_point >= ADJ_CHARPOS)
+  else if (ADJ_BUFPOS == new_point
+	   || (ADJ_ENDPOS && (new_point >= ADJ_BUFPOS)
 	       && (new_point <= ADJ_ENDPOS)))
     {
       w->last_point_x[CURRENT_DISP] = x;
       w->last_point_y[CURRENT_DISP] = y;
-      Fset_marker (w->last_point[CURRENT_DISP], make_int (ADJ_CHARPOS),
+      Fset_marker (w->last_point[CURRENT_DISP], make_int (ADJ_BUFPOS),
 		   w->buffer);
       dl->cursor_elt = x;
       return 1;
@@ -1035,13 +954,13 @@ redisplay_move_cursor (struct window *w, Charbpos new_point, int no_output_end)
       int first = 0;
       int cur_dl, up;
 
-      if (ADJ_CHARPOS < new_point)
+      if (ADJ_BUFPOS < new_point)
 	{
 	  up = 1;
 	  cur_rb = x + 1;
 	  cur_dl = y;
 	}
-      else /* (rb->charbpos + dl->offset) > new_point */
+      else /* (rb->bufpos + dl->offset) > new_point */
 	{
 	  up = 0;
 
@@ -1073,9 +992,9 @@ redisplay_move_cursor (struct window *w, Charbpos new_point, int no_output_end)
 
 	      if (rb->cursor_type != IGNORE_CURSOR
 		  && rb->cursor_type != NO_CURSOR &&
-		  (ADJ_CHARPOS == new_point
-		   || (ADJ_ENDPOS && (new_point >= ADJ_CHARPOS)
-		       && (new_point <= ADJ_CHARPOS))))
+		  (ADJ_BUFPOS == new_point
+		   || (ADJ_ENDPOS && (new_point >= ADJ_BUFPOS)
+		       && (new_point <= ADJ_BUFPOS))))
 		{
 		  rb->cursor_type = CURSOR_ON;
 		  dl->cursor_elt = cur_rb;
@@ -1087,7 +1006,7 @@ redisplay_move_cursor (struct window *w, Charbpos new_point, int no_output_end)
 		  w->last_point_x[CURRENT_DISP] = cur_rb;
 		  w->last_point_y[CURRENT_DISP] = cur_dl;
 		  Fset_marker (w->last_point[CURRENT_DISP],
-			       make_int (ADJ_CHARPOS), w->buffer);
+			       make_int (ADJ_BUFPOS), w->buffer);
 
 		  if (!no_output_end)
 		    {
@@ -1112,7 +1031,7 @@ redisplay_move_cursor (struct window *w, Charbpos new_point, int no_output_end)
     }
   return 0;
 }
-#undef ADJ_CHARPOS
+#undef ADJ_BUFPOS
 #undef ADJ_ENDPOS
 
 /*****************************************************************************
@@ -1258,14 +1177,14 @@ redisplay_unmap_subwindows (struct frame* f, int x, int y, int width, int height
       if (IMAGE_INSTANCE_SUBWINDOW_DISPLAYEDP (ii)
 	  &&
 	  IMAGE_INSTANCE_DISPLAY_X (ii)
-	  + IMAGE_INSTANCE_DISPLAY_WIDTH (ii) > x
+	  + IMAGE_INSTANCE_DISPLAY_WIDTH (ii) > (unsigned) x
 	  &&
-	  IMAGE_INSTANCE_DISPLAY_X (ii) < x + width
+	  IMAGE_INSTANCE_DISPLAY_X (ii) < (unsigned) (x + width)
 	  &&
 	  IMAGE_INSTANCE_DISPLAY_Y (ii)
-	  + IMAGE_INSTANCE_DISPLAY_HEIGHT (ii) > y
+	  + IMAGE_INSTANCE_DISPLAY_HEIGHT (ii) > (unsigned) y
 	  &&
-	  IMAGE_INSTANCE_DISPLAY_Y (ii) < y + height
+	  IMAGE_INSTANCE_DISPLAY_Y (ii) < (unsigned) (y + height)
 	  &&
 	  !EQ (XCAR (rest), ignored_window))
 	{
@@ -1280,9 +1199,7 @@ redisplay_unmap_subwindows (struct frame* f, int x, int y, int width, int height
  Potentially subwindows from the area in the box defined by the given
  parameters.
  ****************************************************************************/
-void
-redisplay_unmap_subwindows_maybe (struct frame *f, int x, int y, int width,
-				  int height)
+void redisplay_unmap_subwindows_maybe (struct frame* f, int x, int y, int width, int height)
 {
   if (!NILP (XWEAK_LIST_LIST (FRAME_SUBWINDOW_CACHE (f))))
     {
@@ -1290,9 +1207,8 @@ redisplay_unmap_subwindows_maybe (struct frame *f, int x, int y, int width,
     }
 }
 
-static void
-redisplay_unmap_subwindows_except_us (struct frame *f, int x, int y, int width,
-				      int height, Lisp_Object subwindow)
+static void redisplay_unmap_subwindows_except_us (struct frame* f, int x, int y, int width,
+						  int height, Lisp_Object subwindow)
 {
   if (!NILP (XWEAK_LIST_LIST (FRAME_SUBWINDOW_CACHE (f))))
     {
@@ -1330,7 +1246,7 @@ redisplay_output_subwindow (struct window *w,
   if (!redisplay_normalize_glyph_area (db, dga))
     return;
 
-  window = wrap_window (w);
+  XSETWINDOW (window, w);
 
   /* Clear the area the subwindow is going into. */
   redisplay_clear_clipped_region (window, findex,
@@ -1401,7 +1317,7 @@ redisplay_output_layout (Lisp_Object domain,
 {
   Lisp_Image_Instance *p = XIMAGE_INSTANCE (image_instance);
   Lisp_Object rest, window = DOMAIN_WINDOW (domain);
-  Ichar_dynarr *buf = Dynarr_new (Ichar);
+  Emchar_dynarr *buf;
   struct window *w = XWINDOW (window);
   struct device *d = DOMAIN_XDEVICE (domain);
   int layout_height, layout_width;
@@ -1418,6 +1334,8 @@ redisplay_output_layout (Lisp_Object domain,
   if (!redisplay_normalize_glyph_area (db, dga))
     return;
 
+  buf = Dynarr_new (Emchar);
+
   /* Highly dodgy optimization. We want to only output the whole
      layout if we really have to. */
   if (!IMAGE_INSTANCE_OPTIMIZE_OUTPUT (p)
@@ -1485,7 +1403,7 @@ redisplay_output_layout (Lisp_Object domain,
   LIST_LOOP (rest, IMAGE_INSTANCE_LAYOUT_CHILDREN (p))
     {
       Lisp_Object child = glyph_image_instance (XCAR (rest), image_instance,
-						ERROR_ME_DEBUG_WARN, 1);
+						ERROR_ME_NOT, 1);
 
       struct display_box cdb;
       /* For losing HP-UX */
@@ -1544,12 +1462,12 @@ redisplay_output_layout (Lisp_Object domain,
 			unsigned char charsets[NUM_LEADING_BYTES];
 			struct face_cachel *cachel = WINDOW_FACE_CACHEL (w, findex);
 
-			find_charsets_in_ibyte_string (charsets,
+			find_charsets_in_bufbyte_string (charsets,
 							 XSTRING_DATA (string),
 							 XSTRING_LENGTH (string));
 			ensure_face_cachel_complete (cachel, window, charsets);
 
-			convert_ibyte_string_into_ichar_dynarr
+			convert_bufbyte_string_into_emchar_dynarr
 			  (XSTRING_DATA (string), XSTRING_LENGTH (string), buf);
 
 			redisplay_normalize_display_box (&cdb, &cdga);
@@ -1604,7 +1522,7 @@ redisplay_output_layout (Lisp_Object domain,
 
 		case IMAGE_POINTER:
 		default:
-		  abort ();
+		  ABORT ();
 		}
 	    }
 	  IMAGE_INSTANCE_OPTIMIZE_OUTPUT (childii) = 0;
@@ -1634,8 +1552,8 @@ redisplay_output_pixmap (struct window *w,
   struct frame *f = XFRAME (w->frame);
   struct device *d = XDEVICE (f->device);
   Lisp_Image_Instance *p = XIMAGE_INSTANCE (image_instance);
-  Lisp_Object window = wrap_window (w);
-
+  Lisp_Object window;
+  XSETWINDOW (window, w);
 
   dga->height = IMAGE_INSTANCE_PIXMAP_HEIGHT (p);
   dga->width = IMAGE_INSTANCE_PIXMAP_WIDTH (p);
@@ -1716,7 +1634,7 @@ redisplay_clear_region (Lisp_Object locale, face_index findex, int x, int y,
       f = XFRAME (locale);
     }
   else
-    abort ();
+    ABORT ();
 
   d = XDEVICE (f->device);
 
@@ -2068,8 +1986,8 @@ redisplay_calculate_display_boxes (struct display_line *dl, int xpos,
 void
 redisplay_clear_top_of_window (struct window *w)
 {
-  Lisp_Object window = wrap_window (w);
-
+  Lisp_Object window;
+  XSETWINDOW (window, w);
 
   if (!NILP (Fwindow_highest_p (window)))
     {
@@ -2120,7 +2038,7 @@ redisplay_clear_to_window_end (struct window *w, int ypos1, int ypos2)
 	  layout_bounds bounds;
 
 	  bounds = calculate_display_line_boundaries (w, bflag);
-	  window = wrap_window (w);
+	  XSETWINDOW (window, w);
 
 	  if (window_is_leftmost (w))
 	    redisplay_clear_region (window, DEFAULT_INDEX, FRAME_LEFT_BORDER_START (f),
@@ -2230,10 +2148,10 @@ redisplay_update_line (struct window *w, int first_line, int last_line,
 
   while (first_line <= last_line)
     {
-      Charcount old_len = (Dynarr_atp (cdla, first_line)->end_charpos -
-			   Dynarr_atp (cdla, first_line)->charpos);
-      Charcount new_len = (Dynarr_atp (ddla, first_line)->end_charpos -
-			   Dynarr_atp (ddla, first_line)->charpos);
+      Charcount old_len = (Dynarr_atp (cdla, first_line)->end_bufpos -
+			   Dynarr_atp (cdla, first_line)->bufpos);
+      Charcount new_len = (Dynarr_atp (ddla, first_line)->end_bufpos -
+			   Dynarr_atp (ddla, first_line)->bufpos);
 
       assert (Dynarr_length (cdla) == Dynarr_length (ddla));
 
@@ -2492,177 +2410,6 @@ redisplay_output_window (struct window *w)
 #endif
 }
 
-/*****************************************************************************
- redisplay_redraw_exposed_window
-
- Given a bounding box for an area that needs to be redrawn, determine
- what parts of what lines are contained within and re-output their
- contents.
- ****************************************************************************/
-static void
-redisplay_redraw_exposed_window (struct window *w, int x, int y, int width,
-				 int height)
-{
-  struct frame *f = XFRAME (w->frame);
-  int line;
-  int start_x, start_y, end_x, end_y;
-  int orig_windows_structure_changed;
-
-  display_line_dynarr *cdla = window_display_lines (w, CURRENT_DISP);
-
-  if (!NILP (w->vchild))
-    {
-      redisplay_redraw_exposed_windows (w->vchild, x, y, width, height);
-      return;
-    }
-  else if (!NILP (w->hchild))
-    {
-      redisplay_redraw_exposed_windows (w->hchild, x, y, width, height);
-      return;
-    }
-
-  /* If the window doesn't intersect the exposed region, we're done here. */
-  if (x >= WINDOW_RIGHT (w) || (x + width) <= WINDOW_LEFT (w)
-      || y >= WINDOW_BOTTOM (w) || (y + height) <= WINDOW_TOP (w))
-    {
-      return;
-    }
-  else
-    {
-      start_x = max (WINDOW_LEFT (w), x);
-      end_x = min (WINDOW_RIGHT (w), (x + width));
-      start_y = max (WINDOW_TOP (w), y);
-      end_y = min (WINDOW_BOTTOM (w), y + height);
-
-      /* We do this to make sure that the 3D modelines get redrawn if
-         they are in the exposed region. */
-      orig_windows_structure_changed = f->windows_structure_changed;
-      f->windows_structure_changed = 1;
-    }
-
-  /* #### Not in GTK or MS Windows.  I think is because of toolbars, which
-     are handled as widgets in GTK and MS Windows, but drawn ourselves in
-     X.  For the moment I'm leaving this in, if it causes problems we have
-     some device method indicating whether we're drawing our own
-     toolbars. */
-  redisplay_clear_top_of_window (w);
-  if (window_needs_vertical_divider (w))
-    {
-      FRAMEMETH (f, output_vertical_divider, (w, 0));
-    }
-
-  for (line = 0; line < Dynarr_length (cdla); line++)
-    {
-      struct display_line *cdl = Dynarr_atp (cdla, line);
-      int top_y = DISPLAY_LINE_YPOS (cdl);
-      int bottom_y = DISPLAY_LINE_YPOS (cdl) + DISPLAY_LINE_HEIGHT (cdl);
-
-      if (bottom_y >= start_y)
-	{
-	  if (top_y > end_y)
-	    {
-	      if (line == 0)
-		continue;
-	      else
-		break;
-	    }
-	  else
-	    {
-	      output_display_line (w, 0, cdla, line, start_x, end_x);
-	    }
-	}
-    }
-
-  f->windows_structure_changed = orig_windows_structure_changed;
-
-  /* If there have never been any face cache_elements created, then this
-     expose event doesn't actually have anything to do. */
-  if (Dynarr_largest (w->face_cachels))
-    redisplay_clear_bottom_of_window (w, cdla, start_y, end_y);
-
-#ifdef HAVE_SCROLLBARS
-  MAYBE_FRAMEMETH (f, redisplay_deadbox, (w, x, y, width, height));
-#endif
-}
-
-
-/*****************************************************************************
- redisplay_redraw_exposed_windows
-
- For each window beneath the given window in the window hierarchy,
- ensure that it is redrawn if necessary after an Expose event.
- ****************************************************************************/
-static void
-redisplay_redraw_exposed_windows (Lisp_Object window, int x, int y, int width,
-				  int height)
-{
-  for (; !NILP (window); window = XWINDOW (window)->next)
-    redisplay_redraw_exposed_window (XWINDOW (window), x, y, width, height);
-}
-
-static void
-redisplay_redraw_exposed_area_1 (Lisp_Object arg)
-{
-  assert (!in_display);
-  redisplay_redraw_exposed_area (XFRAME (X1ST (arg)),
-				 XINT (X2ND (arg)),
-				 XINT (X3RD (arg)),
-				 XINT (X4TH (arg)),
-				 XINT (X5TH (arg)));
-  free_list (arg);
-}
-
-/*****************************************************************************
- redisplay_redraw_exposed_area
-
- For each window on the given frame, ensure that any area in the
- Exposed area is redrawn.
- ****************************************************************************/
-void
-redisplay_redraw_exposed_area (struct frame *f, int x, int y, int width,
-			       int height)
-{
-  int depth;
-
-  if (in_display)
-    {
-      /* Not safe to do it now, so delay it */
-      register_post_redisplay_action (redisplay_redraw_exposed_area_1,
-				      list5 (wrap_frame (f), make_int (x),
-					     make_int (y), make_int (width),
-					     make_int (height)));
-      return;
-    }
-
-  depth = enter_redisplay_critical_section ();
-
-  MAYBE_FRAMEMETH (f, frame_output_begin, (f));
-
-  /* If any window on the frame has had its face cache reset then the
-     redisplay structures are effectively invalid.  If we attempt to
-     use them we'll blow up.  We mark the frame as changed to ensure
-     that redisplay will do a full update.  This probably isn't
-     necessary but it can't hurt. */
-#ifdef HAVE_TOOLBARS
-  /* #### We would rather put these off as well but there is currently
-     no combination of flags which will force an unchanged toolbar to
-     redraw anyhow. */
-  MAYBE_FRAMEMETH (f, redraw_exposed_toolbars, (f, x, y, width, height));
-#endif
-  redraw_exposed_gutters (f, x, y, width, height);
-
-  if (!f->window_face_cache_reset)
-    {
-      redisplay_redraw_exposed_windows (f->root_window, x, y, width, height);
-      /* #### Why not call this always? */
-      MAYBE_FRAMEMETH (f, frame_output_end, (f));
-    }
-  else
-    MARK_FRAME_CHANGED (f);
-
-  exit_redisplay_critical_section (depth);
-}
-
 /*****************************************************************************
  bevel_modeline
 
