@@ -1,7 +1,7 @@
 /* Communication module for TTY terminals.
    Copyright (C) 1994, 1995 Board of Trustees, University of Illinois.
    Copyright (C) 1995 Sun Microsystems, Inc.
-   Copyright (C) 1995, 1996, 2002 Ben Wing.
+   Copyright (C) 1995, 1996 Ben Wing.
    Copyright (C) 1996 Chuck Thompson.
 
 This file is part of XEmacs.
@@ -33,21 +33,17 @@ Boston, MA 02111-1307, USA.  */
 #include "lisp.h"
 
 #include "buffer.h"
-#include "device-impl.h"
+#include "console-tty.h"
 #include "events.h"
 #include "faces.h"
-#include "frame-impl.h"
+#include "frame.h"
 #include "glyphs.h"
 #include "lstream.h"
+#include "objects-tty.h"
 #include "redisplay.h"
 #include "sysdep.h"
 #include "window.h"
 
-#include "console-tty-impl.h"
-#include "objects-tty-impl.h"
-
-#include "syssignal.h"
-
 /* These headers #define all kinds of common words like "columns"...
    What a bunch of losers.  If we were to include them, we'd have to
    include them last to prevent them from messing up our own header
@@ -80,14 +76,14 @@ EXTERN_C void tputs (const char *, int, void (*)(int));
   } while (0)
 #define OUTPUT1_IF(c, a) OUTPUTN_IF (c, a, 1)
 
-static void tty_output_ichar_dynarr (struct window *w,
+static void tty_output_emchar_dynarr (struct window *w,
 				      struct display_line *dl,
-				      Ichar_dynarr *buf, int xpos,
+				      Emchar_dynarr *buf, int xpos,
 				      face_index findex,
 				      int cursor);
-static void tty_output_ibyte_string (struct window *w,
+static void tty_output_bufbyte_string (struct window *w,
 				       struct display_line *dl,
-				       Ibyte *str, Bytecount len,
+				       Bufbyte *str, Bytecount len,
 				       int xpos, face_index findex,
 				       int cursor);
 static void tty_turn_on_face (struct window *w, face_index findex);
@@ -103,13 +99,13 @@ static void term_get_fkeys (Lisp_Object keymap, char **address);
  Non-Mule tty's don't have fonts (that we use at least), so everything
  is considered to be fixed width -- in other words, we return LEN.
  Under Mule, however, a character can still cover more than one
- column, so we use ichar_string_displayed_columns().
+ column, so we use emchar_string_displayed_columns().
  ****************************************************************************/
 static int
-tty_text_width (struct frame *f, struct face_cachel *cachel, const Ichar *str,
+tty_text_width (struct frame *f, struct face_cachel *cachel, const Emchar *str,
 		Charcount len)
 {
-  return ichar_string_displayed_columns (str, len);
+  return emchar_string_displayed_columns (str, len);
 }
 
 /*****************************************************************************
@@ -200,7 +196,7 @@ tty_output_display_block (struct window *w, struct display_line *dl, int block,
 			  int cursor_height)
 {
   struct frame *f = XFRAME (w->frame);
-  Ichar_dynarr *buf = Dynarr_new (Ichar);
+  Emchar_dynarr *buf;
 
   struct display_block *db = Dynarr_atp (dl->display_blocks, block);
   rune_dynarr *rba = db->runes;
@@ -226,7 +222,7 @@ tty_output_display_block (struct window *w, struct display_line *dl, int block,
   if (end < 0)
     end = Dynarr_length (rba);
 
-  Dynarr_reset (buf);
+  buf = Dynarr_new (Emchar);
 
   while (elt < end && Dynarr_atp (rba, elt)->xpos < start_pixpos)
     {
@@ -251,7 +247,7 @@ tty_output_display_block (struct window *w, struct display_line *dl, int block,
 	{
 	  if (Dynarr_length (buf))
 	    {
-	      tty_output_ichar_dynarr (w, dl, buf, xpos, findex, 0);
+	      tty_output_emchar_dynarr (w, dl, buf, xpos, findex, 0);
 	      xpos = rb->xpos;
 	    }
 	  Dynarr_reset (buf);
@@ -266,7 +262,7 @@ tty_output_display_block (struct window *w, struct display_line *dl, int block,
 		  /* Clear in case a cursor was formerly here. */
 
 		  Dynarr_add (buf, ' ');
-		  tty_output_ichar_dynarr (w, dl, buf, rb->xpos,
+		  tty_output_emchar_dynarr (w, dl, buf, rb->xpos,
 					    DEFAULT_INDEX, 0);
 		  Dynarr_reset (buf);
 
@@ -279,7 +275,7 @@ tty_output_display_block (struct window *w, struct display_line *dl, int block,
 		  /* There is not a distinct eol cursor on tty's. */
 
 		  Dynarr_add (buf, rb->object.chr.ch);
-		  tty_output_ichar_dynarr (w, dl, buf, xpos, findex, 0);
+		  tty_output_emchar_dynarr (w, dl, buf, xpos, findex, 0);
 		  Dynarr_reset (buf);
 
 		  cmgoto (f, dl->ypos - 1, xpos);
@@ -293,7 +289,7 @@ tty_output_display_block (struct window *w, struct display_line *dl, int block,
              modeline and this will suffice for that. */
 	  else if (rb->type == RUNE_BLANK || rb->type == RUNE_HLINE)
 	    {
-	      Ichar ch_to_add;
+	      Emchar ch_to_add;
 	      int size = rb->width;
 
 	      if (rb->type == RUNE_BLANK)
@@ -303,7 +299,7 @@ tty_output_display_block (struct window *w, struct display_line *dl, int block,
 
 	      while (size--)
 		Dynarr_add (buf, ch_to_add);
-	      tty_output_ichar_dynarr (w, dl, buf, rb->xpos, findex, 0);
+	      tty_output_emchar_dynarr (w, dl, buf, rb->xpos, findex, 0);
 
 	      if (xpos >= cursor_start
 		  && cursor_start < xpos + Dynarr_length (buf))
@@ -327,9 +323,9 @@ tty_output_display_block (struct window *w, struct display_line *dl, int block,
 	      Lisp_Object window;
 	      Lisp_Object instance;
 
-	      window = wrap_window (w);
+	      XSETWINDOW (window, w);
 	      instance = glyph_image_instance (rb->object.dglyph.glyph,
-					       window, ERROR_ME_DEBUG_WARN, 1);
+					       window, ERROR_ME_NOT, 1);
 
 	      if (IMAGE_INSTANCEP (instance))
 		{
@@ -349,7 +345,7 @@ tty_output_display_block (struct window *w, struct display_line *dl, int block,
 		    case IMAGE_TEXT:
 		    case IMAGE_POINTER:
 		    default:
-		      abort ();
+		      ABORT ();
 		    }
 		  IMAGE_INSTANCE_OPTIMIZE_OUTPUT
 		    (XIMAGE_INSTANCE (instance)) = 0;
@@ -359,12 +355,12 @@ tty_output_display_block (struct window *w, struct display_line *dl, int block,
 	      elt++;
 	    }
 	  else
-	    abort ();
+	    ABORT ();
 	}
     }
 
   if (Dynarr_length (buf))
-    tty_output_ichar_dynarr (w, dl, buf, xpos, findex, 0);
+    tty_output_emchar_dynarr (w, dl, buf, xpos, findex, 0);
   Dynarr_free (buf);
 
 }
@@ -479,8 +475,9 @@ tty_clear_to_window_end (struct window *w, int ypos1, int ypos2)
     }
   else
     {
-      Lisp_Object window = wrap_window (w);
+      Lisp_Object window;
 
+      XSETWINDOW (window, w);
       redisplay_clear_region (window, DEFAULT_INDEX, x, ypos1, width, ypos2 - ypos1);
     }
 }
@@ -520,8 +517,8 @@ tty_clear_frame (struct frame *f)
 }
 
 static void
-tty_output_ibyte_string (struct window *w, struct display_line *dl,
-			   Ibyte *str, Bytecount len, int xpos,
+tty_output_bufbyte_string (struct window *w, struct display_line *dl,
+			   Bufbyte *str, Bytecount len, int xpos,
 			   face_index findex, int cursor)
 {
   struct frame *f = XFRAME (w->frame);
@@ -534,53 +531,53 @@ tty_output_ibyte_string (struct window *w, struct display_line *dl,
   tty_turn_on_face (w, findex);
 
   send_string_to_tty_console (c, str, len);
-  TTY_INC_CURSOR_X (c, ibyte_string_displayed_columns (str, len));
+  TTY_INC_CURSOR_X (c, bufbyte_string_displayed_columns (str, len));
 
   /* Turn the face properties back off. */
   tty_turn_off_face (w, findex);
 }
 
-static Ibyte_dynarr *tty_output_ichar_dynarr_dynarr;
+static Bufbyte_dynarr *tty_output_emchar_dynarr_dynarr;
 
 /*****************************************************************************
- tty_output_ichar_dynarr
+ tty_output_emchar_dynarr
 
  Given a string and a starting position, output that string in the
  given face.  If cursor is true, draw a cursor around the string.
  ****************************************************************************/
 static void
-tty_output_ichar_dynarr (struct window *w, struct display_line *dl,
-			  Ichar_dynarr *buf, int xpos, face_index findex,
+tty_output_emchar_dynarr (struct window *w, struct display_line *dl,
+			  Emchar_dynarr *buf, int xpos, face_index findex,
 			  int cursor)
 {
-  if (!tty_output_ichar_dynarr_dynarr)
-    tty_output_ichar_dynarr_dynarr = Dynarr_new (Ibyte);
+  if (!tty_output_emchar_dynarr_dynarr)
+    tty_output_emchar_dynarr_dynarr = Dynarr_new (Bufbyte);
   else
-    Dynarr_reset (tty_output_ichar_dynarr_dynarr);
+    Dynarr_reset (tty_output_emchar_dynarr_dynarr);
 
-  convert_ichar_string_into_ibyte_dynarr (Dynarr_atp (buf, 0),
+  convert_emchar_string_into_bufbyte_dynarr (Dynarr_atp (buf, 0),
 					    Dynarr_length (buf),
-					    tty_output_ichar_dynarr_dynarr);
+					    tty_output_emchar_dynarr_dynarr);
 
-  tty_output_ibyte_string (w, dl,
-			     Dynarr_atp (tty_output_ichar_dynarr_dynarr, 0),
-			     Dynarr_length (tty_output_ichar_dynarr_dynarr),
+  tty_output_bufbyte_string (w, dl,
+			     Dynarr_atp (tty_output_emchar_dynarr_dynarr, 0),
+			     Dynarr_length (tty_output_emchar_dynarr_dynarr),
 			     xpos, findex, cursor);
 }
 
 #if 0
 
-static Ibyte_dynarr *sidcs_dynarr;
+static Bufbyte_dynarr *sidcs_dynarr;
 
 static void
 substitute_in_dynamic_color_string (Lisp_Object spec, Lisp_Object string)
 {
   int i;
-  Ibyte *specdata = XSTRING_DATA   (spec);
+  Bufbyte *specdata = XSTRING_DATA   (spec);
   Bytecount speclen = XSTRING_LENGTH (spec);
 
   if (!sidcs_dynarr)
-    sidcs_dynarr = Dynarr_new (Ibyte);
+    sidcs_dynarr = Dynarr_new (Bufbyte);
   else
     Dynarr_reset (sidcs_dynarr);
 
@@ -609,7 +606,7 @@ static void
 set_foreground_to (struct console *c, Lisp_Object sym)
 {
   Lisp_Object result;
-  Ibyte *escseq = 0;
+  Bufbyte *escseq = 0;
   Bytecount escseqlen = 0;
 
   result = assq_no_quit (sym, Vtty_color_alist);
@@ -639,7 +636,7 @@ static void
 set_background_to (struct console *c, Lisp_Object sym)
 {
   Lisp_Object result;
-  Ibyte *escseq = 0;
+  Bufbyte *escseq = 0;
   Bytecount escseqlen = 0;
 
   result = assq_no_quit (sym, Vtty_color_alist);
@@ -799,7 +796,7 @@ tty_turn_on_frame_face (struct frame *f, Lisp_Object face)
   Lisp_Object frame;
   struct console *c = XCONSOLE (FRAME_CONSOLE (f));
 
-  frame = wrap_frame (f);
+  XSETFRAME (frame, f);
   tty_turn_on_face_1 (c,
 		      FACE_HIGHLIGHT_P (face, frame),
 		      FACE_BLINKING_P (face, frame),
@@ -823,7 +820,7 @@ tty_turn_off_frame_face (struct frame *f, Lisp_Object face)
   Lisp_Object frame;
   struct console *c = XCONSOLE (FRAME_CONSOLE (f));
 
-  frame = wrap_frame (f);
+  XSETFRAME (frame, f);
 
   if (FACE_REVERSE_P (face, frame))
     {
@@ -867,6 +864,9 @@ set_tty_modes (struct console *c)
   OUTPUT1_IF (c, TTY_SD (c).init_motion);
   OUTPUT1_IF (c, TTY_SD (c).cursor_visible);
   OUTPUT1_IF (c, TTY_SD (c).keypad_on);
+
+  if (TTY_FLAGS (c).auto_margins)
+    OUTPUT1_IF (c, TTY_SD (c).disable_auto_margins);
 }
 
 /*****************************************************************************
@@ -885,6 +885,9 @@ reset_tty_modes (struct console *c)
   OUTPUT1_IF (c, TTY_SD (c).cursor_normal);
   OUTPUT1_IF (c, TTY_SD (c).end_motion);
 
+  if (TTY_FLAGS (c).auto_margins)
+    OUTPUT1_IF (c, TTY_SD (c).enable_auto_margins);
+
   {
     Lisp_Object frm = CONSOLE_SELECTED_FRAME (c);
 
@@ -1188,12 +1191,14 @@ init_tty_for_redisplay (struct device *d, char *terminal_type)
       TTY_SD (c).audio_bell = "\07";
     }
 
-  TTY_SD (c).cursor_visible = tgetstr ("ve", &bufptr);
-  TTY_SD (c).cursor_normal = tgetstr ("vs", &bufptr);
+  TTY_SD (c).cursor_visible = tgetstr ("vs", &bufptr);
+  TTY_SD (c).cursor_normal = tgetstr ("ve", &bufptr);
   TTY_SD (c).init_motion = tgetstr ("ti", &bufptr);
   TTY_SD (c).end_motion = tgetstr ("te", &bufptr);
   TTY_SD (c).keypad_on = tgetstr ("ks", &bufptr);
   TTY_SD (c).keypad_off = tgetstr ("ke", &bufptr);
+  TTY_SD (c).disable_auto_margins = tgetstr ("RA", &bufptr);
+  TTY_SD (c).enable_auto_margins = tgetstr ("SA", &bufptr);
 
 
   /*
@@ -1206,6 +1211,7 @@ init_tty_for_redisplay (struct device *d, char *terminal_type)
   TTY_FLAGS (c).memory_below_frame = tgetflag ("db");
   TTY_FLAGS (c).standout_width = tgetnum ("sg");
   TTY_FLAGS (c).underline_width = tgetnum ("ug");
+  TTY_FLAGS (c).auto_margins = tgetflag ("am");
 
   if (TTY_FLAGS (c).standout_width == -1)
     TTY_FLAGS (c).standout_width = 0;
