@@ -2,8 +2,8 @@
 emacs_is_beta=
 emacs_major_version=21
 emacs_minor_version=4
-emacs_beta_version=23
-xemacs_codename="Moral Majority"
+emacs_beta_version=24
+xemacs_codename="Standard C"
 emacs_kit_version=
 infodock_major_version=4
 infodock_minor_version=0
